/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.battlefieldelements;

import java.awt.Graphics;
import java.awt.Rectangle;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

/**
 * Mother class of all the battlefield elements
 * @author Farfpeuf
 * 
 */
public abstract class Element {

	//constants used for property change events
	
	public static final String BOUNDS = "bounds";

	public static final String IMAGE = "image";

	//property change delegate
	
	protected PropertyChangeSupport support;

	
	/**
	 * get x game coordinate
	 * @return x game coordinate
	 */
	public abstract int getX();

	/**
	 * get y game coordinate
	 * @return y game coordinate
	 */
	public abstract int getY();

	/**
	 * get bounds 
	 * @return bounds
	 */
	public abstract Rectangle getBounds();

	/**
	 * paint the element image
	 * @param g graphics
	 * @param dx x offset
	 * @param dy y offset
	 * @param scale view scale
	 */
	public abstract void paintImage(Graphics g, int dx, int dy, int scale);

	/**
	 * create a new Element
	 *
	 */
	public Element() {
		support = new PropertyChangeSupport(this);
	}

	//------------------PropertyChange delegation----------
	
	public void addPropertyChangeListener(String arg0,
			PropertyChangeListener arg1) {
		support.addPropertyChangeListener(arg0, arg1);
	}

	public void firePropertyChange(String arg0, boolean arg1, boolean arg2) {
		support.firePropertyChange(arg0, arg1, arg2);
	}

	public void firePropertyChange(String arg0, int arg1, int arg2) {
		support.firePropertyChange(arg0, arg1, arg2);
	}

	public void firePropertyChange(String arg0, Object arg1, Object arg2) {
		support.firePropertyChange(arg0, arg1, arg2);
	}

	public void removePropertyChangeListener(String arg0,
			PropertyChangeListener arg1) {
		support.removePropertyChangeListener(arg0, arg1);
	}

	public void addPropertyChangeListener(PropertyChangeListener listener) {
		support.addPropertyChangeListener(listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener listener) {
		support.removePropertyChangeListener(listener);
	}

}
