/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.battlefieldelements;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.Hashtable;

import fb.util.FBImage;
import fb.util.Resources;
/**
 * a vertical set (tree, building, ...)
 * @author Farfpeuf
 */
public class VerticalSet extends Element {
	//position on the battlefield
	private int x, y;
	//image used for painting
	private FBImage image;
	//vertical sets available images
	private static Hashtable imageTable = new Hashtable();
	/**
	 * get a set image
	 * @param type type of set
	 * @return the image singleton
	 */
	public static FBImage getFantasyImage(String type) {
		FBImage image = (FBImage) imageTable.get(type);
		if (null == image) {
			image = new FBImage(Resources.getVerticalSetImage(type));
			imageTable.put(type, image);
		}
		return image;
	}
	/**
	 * create a set of specified type
	 * @param position position on the battlefield
	 * @param type type of set
	 */
	public VerticalSet(Point position, String type) {
		super();
		x = (int) position.getX();
		y = (int) position.getY();
		image = getFantasyImage(type);
	}

	/**
	 * @return Returns the x.
	 */
	public int getX() {
		return x;
	}
	/**
	 * @return Returns the y.
	 */
	public int getY() {
		return y;
	}
	/**
	 * paint the set on the battlefield
	 */
	public void paintImage(Graphics g,int dx,int dy,int scale) {
			image.draw(g, x, y, scale,dx,dy);
	}
	
	/**
	 * get the set bounds
	 * 
	 */
	public Rectangle getBounds(){
		int imageWidth=image.getGameWidth();
		int imageHeight=image.getGameHeight();

		int rx=x- imageWidth/ 2;
		int ry=y- imageHeight/2;

		return new Rectangle(rx,ry,imageWidth,imageHeight);
	}
	
}