/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.campaign;

import java.awt.Point;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import fb.army.hierarchy.Army;
import fb.army.hierarchy.Force;
import fb.artificialintelligence.AIConstants;
import fb.artificialintelligence.ArmyRiser;
import fb.artificialintelligence.WeightChooser;
import fb.gui.panels.MapChooser;
import fb.main.Battle;
import fb.main.FB;
import fb.main.LoadingException;
import fb.player.PlayersHandler;
import fb.util.Resources;
import fb.util.StringTools;

/**
 * a fantsy battlefields campaign game
 * 
 * @author farfpeuf
 * 
 */
public class Campaign implements PropertyChangeListener {

	// tag used to separate game data from campaign data when campaign is saved
	// to a file
	public static final String GAME_TAG = "#game#";

	// PropertyChangeListener properties
	public static final String WAITING_PROPERTY = "waiting";

	public static final String SELECTED_ARM_PROPERTY = "selected army";

	public static final String CURRENT_FORCE_PROPERTY = "force";

	public static final String MVT_ZONE_PROPERTY = "zone";

	// singleton
	private static Campaign instance;

	// campaign budget
	private int budget = 2000;

	// campaign forces
	private Force[] forces;

	// current selected army
	private Army selectedArmy;

	// current army movement zone
	private List movementZone;

	// campaign players handler
	private PlayersHandler playersHandler;

	// current force id
	private int currentForce = 0;

	// PropertyChange delegate
	private PropertyChangeSupport support;

	// waiting for player action ?
	private boolean waiting;

	// map city/player controlling the city
	private Hashtable cityControls;

	/**
	 * wait or stop waiting for map movements animation
	 * 
	 * @param wait
	 *            wait or stop waiting if false
	 */
	public void waitAnimation(boolean wait) {
		if (this.waiting != wait) {
			boolean oldWait = this.waiting;
			this.waiting = wait;
			firePropertyChange(WAITING_PROPERTY, oldWait, wait);

		}
	}

	/**
	 * change current force
	 * 
	 */
	public void changeForce() {
		int oldForce = currentForce;
		int i = currentForce + 1;
		if (i == forces.length) {
			i = 0;
		}

		currentForce = i;
		manageIncome();
		List arms = forces[currentForce].getArmies();
		Army arm;
		for (int j = 0; j < arms.size(); j++) {
			arm = (Army) arms.get(j);
			arm.resetRemainingMove();
		}

		setSelectedArmy(null);
		CampaignStatePanel.getInstance().setPlayer(
				forces[currentForce].getColor(),
				forces[currentForce].getPlayerName(),
				forces[currentForce].isAi(), forces[currentForce].isHandled());

		firePropertyChange(CURRENT_FORCE_PROPERTY, oldForce, currentForce);
		boolean end = testEnd();
		if (!end && currentForce().isHandled() && currentForce().isAi()) {
			handleAI();
		}
	}

	/**
	 * handle artificial intelligence players
	 * 
	 */
	private void handleAI() {
		if ((currentForce().getBudget() > 500 || currentForce().getChildCount() == 0)
				&& controlsCapital(currentForce().getSide())) {
			riseAI();
		}
		Army arm;
		for (int i = currentForce().getChildCount() - 1; i >= 0; i--) {
			arm = (Army) currentForce().getChildAt(i);
			setSelectedArmy(arm);
			moveAI(arm);
		}
		changeForce();
	}

	/**
	 * calculate budget for a new army for an AI player
	 * 
	 * @return the budget
	 */
	private int getAIArmyBudget() {

		int newArmBudget;

		Army thisStrongest = currentForce().getStrongestArm();
		Army ennemiesStrongest = null, tmp;

		int nbEnnemies = 0, nbEnnemyArmies = 0, ennemiesBudget = 0;

		for (int i = 0; i < forces.length; i++) {
			if (forces[i].getSide() != currentForce().getSide()) {
				nbEnnemies++;
				nbEnnemyArmies += forces[i].getChildCount();
				ennemiesBudget += forces[i].getBudget();
				tmp = forces[i].getStrongestArm();
				if (ennemiesStrongest == null
						|| tmp.getCost() > ennemiesStrongest.getCost()) {
					ennemiesStrongest = tmp;
				}
			}
		}

		if (nbEnnemyArmies / nbEnnemies > currentForce().getChildCount()) {
			if (currentForce().getBudget() >= ennemiesBudget / nbEnnemies) {
				newArmBudget = (currentForce().getBudget() - ennemiesBudget
						/ nbEnnemies)
						/ (nbEnnemyArmies / nbEnnemies - currentForce()
								.getChildCount());
			} else {
				newArmBudget = currentForce().getBudget()
						/ (nbEnnemyArmies / nbEnnemies - currentForce()
								.getChildCount());
			}
		} else {
			newArmBudget = 1000;
		}

		if (ennemiesStrongest != null
				&& (thisStrongest == null || (thisStrongest.getCost() + 200) < ennemiesStrongest
						.getCost())) {
			newArmBudget = ennemiesStrongest.getCost();
		}

		return Math.min(currentForce().getBudget(), newArmBudget);
	}

	/**
	 * rise an Army for AI player
	 * 
	 */
	private void riseAI() {
		Army newArm = ArmyRiser.riseAI(currentForce(), getAIArmyBudget());
		newArm.resetRemainingMove();
		Point capital = HexagonalMapModel.getInstance().getCapital(
				currentForce().getSide());
		if (capital != null) {
			newArm.initPosition(capital);
		}
		CampaignStatePanel.getInstance().updateForceTab(currentForce());
	}

	/**
	 * handle battle between 2 armies
	 * 
	 * @param attacker
	 *            the attacker
	 * @param defender
	 *            the defender
	 */
	private synchronized void fight(Army attacker, Army defender) {
		Army[] arms = new Army[2];
		arms[0] = attacker;
		arms[1] = defender;
		attacker.setGmId(0);
		defender.setGmId(1);

		String map = getMap(defender.getPosition());
		MapChooser.getInstance(true).setMap(map);
		FB.getInstance().clearFrame();
		CampaignGameFrame.getInstance().stop();
		FB.getInstance().playGame(arms, false, true);
		try {
			wait();
		} catch (InterruptedException e) {
			FB.exception(e);
		}
	}

	/**
	 * merge 2 friendly armies
	 * 
	 * @param source
	 *            source army
	 * @param dest
	 *            destination army
	 */
	private void merge(Army source, Army dest) {
		if (source == dest) {
			FB.debug("Bad merge");
		} else {

			Force oldForce = source.getForce();
			oldForce.removeArmy(source);
			CampaignStatePanel.getInstance().updateForceTab(oldForce);

			dest.merge(source);
			dest.noRemainingMove();
			CampaignStatePanel.getInstance().updateForceTab(dest.getForce());
		}
	}

	/**
	 * get corresponding battle map for this campaign map location
	 * 
	 * @param location
	 *            the location
	 * @return the battle map name
	 */
	private String getMap(Point location) {

		if (HexagonalMapModel.getInstance().isCapital(location)) {
			return "town";
		} else if (HexagonalMapModel.getInstance().isVillage(location)) {
			return "village";
		}

		char tile = HexagonalMapModel.getInstance().getTile(location.x,
				location.y);
		switch (tile) {
		case 'e':
			List surroundings = Hexagon.getSurrondingCoords(location, 1,
					HexagonalMapModel.getInstance().getBounds());
			Point surrounding;
			char surroundingTile;

			for (int i = 0; i < surroundings.size(); i++) {
				surrounding = (Point) surroundings.get(i);
				surroundingTile = HexagonalMapModel.getInstance().getTile(
						surrounding.x, surrounding.y);

				if (surroundingTile != 'e') {
					return "dolmens";
				}

			}

			return "forest-island";
		case 'b':
			return "oasis";
		case 'a':
			surroundings = Hexagon.getSurrondingCoords(location, 1,
					HexagonalMapModel.getInstance().getBounds());

			for (int i = 0; i < surroundings.size(); i++) {
				surrounding = (Point) surroundings.get(i);
				surroundingTile = HexagonalMapModel.getInstance().getTile(
						surrounding.x, surrounding.y);

				if (surroundingTile == 'e') {
					return "river";
				}

			}
			return "lakes";
		case 'h':
			return "forest";
		case 'd':
			return "snow-lake";
		}

		return "forest";
	}

	/**
	 * move an AI player army
	 * 
	 * @param army
	 *            the ai player army
	 */
	private void moveAI(Army army) {
		Point destination = getAIDestination(army);

		Army oldDestArm = null, destArm = getArmy(destination);
		if (destArm != null && destArm.getSide() != army.getSide()) {

			while (destArm != null && !destination.equals(army.getPosition())) {
				int direction = Hexagon.getDirectDirection(destination, army
						.getPosition());
				destination = Hexagon.getTranslatedCoords(destination, 1,
						direction);
				oldDestArm = destArm;
				destArm = getArmy(destination);
			}
			if (!destination.equals(army.getPosition())) {
				moveArm(army, destination);
				while (army.isMoving()) {
					try {
						Thread.sleep(100);
					} catch (InterruptedException e) {
						FB.exception(e);
					}
				}
			}
			if (oldDestArm != null && oldDestArm.getSide() != army.getSide()) {
				fight(army, oldDestArm);
			}
		} else {
			if (destination != null) {
				moveArm(army, destination);
			}
		}
	}

	/**
	 * calculate ai player army best destination
	 * 
	 * @param arm
	 *            th ai player army
	 * @return the destination point
	 */
	public Point getAIDestination(final Army arm) {

		int nbEnnemies = 0, nbEnnemyArmies = 0;

		for (int i = 0; i < forces.length; i++) {
			if (forces[i].getSide() != currentForce().getSide()) {
				nbEnnemies++;
				nbEnnemyArmies += forces[i].getChildCount();
			}
		}

		final int averageNbEnnemies = nbEnnemyArmies / nbEnnemies;

		Point destination;
		WeightChooser destinationChooser = new WeightChooser(movementZone, 3,
				14) {

			protected int getWeight(Object element) {
				Point dest = (Point) element;
				Army destArm = getArmy(dest);
				Point capital = HexagonalMapModel.getInstance().getCapital(
						arm.getSide());
				int weight = super.getWeight(element);
				if (!controlsCapital(arm.getSide())) {

					if (chosenElement == null
							|| Hexagon.getDirectDistance((Point) chosenElement,
									capital) > Hexagon.getDirectDistance(dest,
									capital)) {
						weight = maxWeight + 1000;
					}
				} else {
					if (HexagonalMapModel.getInstance().isVillage(dest)
							&& !controlsCity(arm.getSide(), dest)) {
						weight += 3;

						if (destArm == null) {
							weight += 10;
						}

					} else if (HexagonalMapModel.getInstance().isCapital(dest)
							&& !controlsCity(arm.getSide(), dest)) {
						weight += 6;
						if (destArm == null) {
							weight += 20;
						}
					}
					if (destArm != null) {
						if (destArm.getSide() == arm.getSide()) {
							int diffNbArmies = arm.getForce().getChildCount()
									- averageNbEnnemies;
							weight += diffNbArmies;

							weight += 4;

							weight -= (arm.getCost() / 100);
						} else {
							float ratio = (float) arm.getCost()
									/ (float) destArm.getCost();
							int diffRatio = (int) ((ratio - 1.0f) * 100.0);

							weight += Math.max(Math.min((diffRatio / 5), 15),
									-15);
							if (getArmy(capital) == null
									&& Hexagon.getDirectDistance(destArm
											.getPosition(), capital) < destArm
											.movement()) {
								weight += 5;
							}
						}

					} else {
						CampaignMovementZoneFinder zoneFinder=new CampaignMovementZoneFinder(dest,arm,HexagonalMapModel.getInstance()
								.getBounds());
						zoneFinder.run();
						List destZone = zoneFinder.getMovementZone();
						zoneFinder.run();
						Point destSurrounding;
						Iterator iter = destZone.iterator();
						while (iter.hasNext()) {
							destSurrounding = ((Point) iter.next());
							if (HexagonalMapModel.getInstance().isVillage(
									destSurrounding)
									|| HexagonalMapModel.getInstance()
											.isCapital(destSurrounding)
									&& !controlsCity(arm.getSide(),
											destSurrounding)) {
								weight += 2;
								if (destArm == null) {
									weight += 5;
								}
								if (!destSurrounding.equals(capital)
										&& HexagonalMapModel.getInstance()
												.isCapital(destSurrounding)) {
									weight += 1;
									if (destArm == null) {
										weight += 2;
									}
								}

							}
						}

					}
				}

				return weight;
			}

		};
		synchronized (destinationChooser) {
			destinationChooser.start();
			try {
				destinationChooser.wait(FB.getInstance().isBatch() ? 200
						: AIConstants.MAX_PATH_SEARCHING_TIME);
			} catch (InterruptedException e) {
				FB.exception(e);
			}
		}
		destination = (Point) destinationChooser.getChosenElement();
		if (destinationChooser.isAlive()) {
			WeightChooser moribund = destinationChooser;
			destinationChooser = null;
			moribund.safeStop();
		}
		return destination;
	}

	/**
	 * calculates incomes for each forces
	 * 
	 */
	private void manageIncome() {
		Enumeration keys = cityControls.keys();
		Point city;
		int income = 0;
		int side;
		while (keys.hasMoreElements()) {
			city = (Point) keys.nextElement();
			side = ((Integer) cityControls.get(city)).intValue();
			if (side == forces[currentForce].getSide()) {
				if (HexagonalMapModel.getInstance().isCapital(city)) {
					income += 30;
				} else {
					income += 20;
				}
			}
		}
		forces[currentForce].receiveIncome(income);
	}

	/**
	 * did all the armies of the current force move ?
	 * 
	 * @return true if they all moved
	 */
	public boolean allMoved() {
		List arms = forces[currentForce].getArmies();
		Army arm;
		for (int j = 0; j < arms.size(); j++) {
			arm = (Army) arms.get(j);
			if (arm.getRemainingMove() > 0) {
				return false;
			}
		}
		return true;
	}

	/**
	 * get singleton instance
	 * 
	 * @return the singleton
	 */
	public static synchronized Campaign getInstance() {
		if (instance == null) {
			instance = new Campaign();
		}
		return instance;
	}

	/**
	 * does current force controls its own capital ?
	 * 
	 * @return true if it does
	 */
	public boolean currentForceControlsCapital() {
		return controlsCapital(forces[currentForce].getSide());
	}

	/**
	 * does specified side controls its own capital ?
	 * 
	 * @param side
	 *            the side
	 * @return true if it does
	 */
	public boolean controlsCapital(int side) {
		Point capital = HexagonalMapModel.getInstance().getCapital(side);
		return controlsCity(side, capital);
	}

	/**
	 * does specified side controls a specified city
	 * 
	 * @param side
	 *            the side
	 * @param city
	 *            the city
	 * @return true if it does
	 */
	public boolean controlsCity(int side, Point city) {
		Integer controller = (Integer) cityControls.get(city);
		if (controller == null) {
			return false;
		}
		return controller.intValue() == side;
	}

	/**
	 * create a new campaign
	 * 
	 */
	private Campaign() {
		support = new PropertyChangeSupport(this);
		playersHandler = new PlayersHandler();
		cityControls = new Hashtable();
	}

	/**
	 * reset the campaign
	 * 
	 */
	public void reset() {

	}

	/**
	 * returns campaign as a string to save it in a file
	 * 
	 * @param saveGame
	 *            also save current game ?
	 * @return the campaign as a string
	 */
	public String saveString(boolean saveGame) {
		String result =

		HexagonalMapModel.getInstance().getModelString() + "#" + budget + "\t"
				+ currentForce + "\t"
				+ ((selectedArmy == null) ? -1 : selectedArmy.getGmId());

		Enumeration keys = cityControls.keys();

		Point city;
		int side;
		while (keys.hasMoreElements()) {
			city = (Point) keys.nextElement();
			side = ((Integer) cityControls.get(city)).intValue();
			result += "\t" + city.x;
			result += "\t" + city.y;
			result += "\t" + side;
		}

		for (int i = 0; i < forces.length; i++) {
			result += Force.FORCE_TAG + forces[i].saveBattleString();
		}

		if (saveGame) {
			result += GAME_TAG + Battle.getInstance().saveCampaignBattleString();
		}

		return result;
	}

	/**
	 * load campaign from a string
	 * 
	 * @param cmpString
	 *            the campaign string
	 * @throws LoadingException
	 */
	public void loadString(String cmpString) throws LoadingException {

		String[] tokens = cmpString.split(Force.FORCE_TAG);

		StringBuffer buffy = new StringBuffer(tokens[0]);

		String mapSt = StringTools.readBuffer(buffy, '#');

		HexagonalMapModel.getInstance().load(mapSt);

		budget = StringTools.readBufferInt(buffy);
		currentForce = StringTools.readBufferInt(buffy);
		int selectedArmyId = StringTools.readBufferInt(buffy);

		Point city;
		int side;

		while (buffy.length() > 0) {
			city = new Point();
			city.x = StringTools.readBufferInt(buffy);
			city.y = StringTools.readBufferInt(buffy);
			side = StringTools.readBufferInt(buffy);
			cityControls.remove(city);
			cityControls.put(city, new Integer(side));
		}

		forces = new Force[tokens.length - 1];

		for (int i = 1; i < tokens.length; i++) {
			forces[i - 1] = new Force(tokens[i]);
		}
		if (selectedArmyId == -1) {
			selectedArmy = null;
		} else {
			selectedArmy = (Army) forces[currentForce]
					.getChildAt(selectedArmyId);
		}

	}

	/**
	 * stop current campaign
	 * 
	 */
	public void stop() {
		forces = null;
		selectedArmy = null;
		movementZone = null;
		cityControls.clear();
	}

	/**
	 * change campaign budget
	 * 
	 * @param budget
	 *            the new budget
	 */
	public void setBudget(int budget) {
		this.budget = budget;
	}

	/**
	 * @return Returns the budget.
	 */
	public int getBudget() {
		return budget;
	}

	/**
	 * @return Returns the forces.
	 */
	public Force[] getForces() {
		return forces;
	}

	/**
	 * @return Returns the playersHandler.
	 */
	public PlayersHandler getPlayersHandler() {
		return playersHandler;
	}

	/**
	 * get current force
	 * @return the current force
	 */
	public Force currentForce() {
		if (forces == null) {
			return null;
		}
		return forces[currentForce];
	}

	/**
	 * set campaign forces
	 * @param forces the forces
	 */
	public void setForces(Force[] forces) {
		this.forces = forces;
		Point capital;
		for (int i = 0; i < forces.length; i++) {
			capital = HexagonalMapModel.getInstance().getCapital(
					forces[i].getSide());
			cityControls.remove(capital);
			cityControls.put(capital, new Integer(forces[i].getSide()));
		}

		changeForce();
	}

	/**
	 * get the ofrces of one specified side
	 * @param side the side
	 * @return a list of forces
	 */
	public List getForces(int side) {
		List result = new ArrayList();
		for (int i = 0; i < forces.length; i++) {
			if (forces[i].getSide() == side) {
				result.add(forces[i]);
			}
		}
		return result;
	}

	/**
	 * get the selected army
	 * @return the current selected army
	 */
	public Army getSelectedArmy() {
		return selectedArmy;
	}

	/**
	 * change current selected army
	 * @param selectedArmy the new army
	 */
	public void setSelectedArmy(Army selectedArmy) {
		if (this.selectedArmy != null) {
			this.selectedArmy.removePropertyChangeListener(Army.POSITION, this);
		}
		Army oldArm = this.selectedArmy;
		this.selectedArmy = selectedArmy;
		firePropertyChange(SELECTED_ARM_PROPERTY, oldArm, selectedArmy);
		if (this.selectedArmy != null) {
			this.selectedArmy.addPropertyChangeListener(Army.POSITION, this);
			setMovementZone(selectedArmy);
		} else {
			setMovementZone((Army) null);
		}

	}

	/**
	 * get current army movement zone
	 * @return the movement zone (list of points)
	 */
	public List getMovementZone() {
		return movementZone;
	}

	/**
	 * handle property changes
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals(Army.POSITION)) {
			if (selectedArmy != null) {
				setMovementZone(selectedArmy);
			} else {
				setMovementZone((Army) null);
			}
		}
	}

	/**
	 * caclculate movement zone for specified army
	 * @param army the army
	 */
	private void setMovementZone(Army army) {
		List mouvementZone = null;
		if (army != null) {

			CampaignMovementZoneFinder zoneFinder=new CampaignMovementZoneFinder(army.getPosition(),army,HexagonalMapModel.getInstance()
					.getBounds());
			
			zoneFinder.run();
			mouvementZone = zoneFinder.getMovementZone();
		}
		/*
		 * List armies; Arm tmpArm; for (int i = 0; i < forces.length; i++) {
		 * armies = forces[i].getArmies(); for (int j = 0; j < armies.size();
		 * j++) { tmpArm = (Arm) armies.get(j);
		 * mouvementZone.remove(tmpArm.getHexaCoord()); } }
		 */
		setMovementZone(mouvementZone);
	}

	/**
	 * change current movement zone
	 * @param movementZone the new movement zone(list of points)
	 */
	private void setMovementZone(List movementZone) {
		List oldZone = this.movementZone;
		this.movementZone = movementZone;
		firePropertyChange(MVT_ZONE_PROPERTY, oldZone, movementZone);
	}

	/**
	 * end current battle
	 * @param winner the battle winner id
	 * @param armies the engaged armies
	 */
	public synchronized void endBattle(int winner, Army[] armies) {
		if (winner == armies[0].getSide()) {

			Point wonField = armies[1].getPosition();

			int direction = Hexagon.getDirectDirection(armies[0].getPosition(),
					armies[1].getPosition());
			Point retreat = Hexagon.getTranslatedCoords(
					armies[1].getPosition(), 1, direction);

			Army destArm = getArmy(retreat);

			if ((!HexagonalMapModel.getInstance().getBounds().contains(retreat))
					|| destArm != null
					&& destArm.getSide() != armies[1].getSide()) {

				List retreats = Hexagon.getSurrondingCoords(retreat, 1,
						HexagonalMapModel.getInstance().getBounds());
				retreat = null;
				Point tmp;
				for (int i = 0; i < retreats.size() && retreat == null; i++) {
					tmp = (Point) retreats.get(i);
					destArm = getArmy(tmp);
					if (destArm == null
							|| destArm.getSide() == armies[1].getSide()) {
						retreat = tmp;
					}

				}

			}
			if (retreat != null) {
				moveArm(armies[1], retreat);
			} else {
				armies[1].destroy();
				if (armies[1] == selectedArmy) {
					setSelectedArmy(null);
				}
			}

			moveArm(armies[0], wonField);

		}
		Force force;
		for (int i = 0; i < armies.length; i++) {
			force = armies[i].getForce();
			if (armies[i].isAnnihilated()) {
				armies[i].destroy();
				if (armies[i] == selectedArmy) {
					setSelectedArmy(null);
				}
			} else {
				armies[i].endBattle(winner == armies[i].getSide());

				if (armies[i].getNumberOfWarriors() <= 1 && winner != armies[i].getSide()) {
					armies[i].destroy();
					if (armies[i] == selectedArmy) {
						setSelectedArmy(null);
					}
				}

			}
			CampaignStatePanel.getInstance().updateForceTab(force);
		}
		testEnd();
		notify();
	}

	/**
	 * test if campaign is ended
	 * @return true if campaign is ended
	 */
	private boolean testEnd() {
		int campaignWinner = winner();
		if (campaignWinner != -1) {
			endCmp(campaignWinner);
			return true;
		}
		return false;
	}

	/**
	 * get army at specified point
	 * @param hexaCoord the point
	 * @return the army or null if none
	 */
	public Army getArmy(Point hexaCoord) {

		List armies;
		Army army;
		for (int i = 0; i < forces.length; i++) {
			armies = forces[i].getArmies();
			for (int j = 0; j < armies.size(); j++) {
				army = (Army) armies.get(j);
				if (army.getPosition().equals(hexaCoord)) {
					return army;
				}

			}
		}
		return null;
	}

	/**
	 * handle click on a specified hexagon
	 * @param hexaClick the hexagon
	 */
	public synchronized void click(Point hexaClick) {

		if (currentForce().isHandled() && !currentForce().isAi()) {

			Army clickedArmy = getArmy(hexaClick);
			if (clickedArmy == selectedArmy) {
				setSelectedArmy(null);
			} else if (clickedArmy == null) {
				List mvtZone = getMovementZone();
				if (mvtZone != null && mvtZone.contains(hexaClick)) {
					moveArm(selectedArmy, hexaClick);
					if (allMoved()) {
						changeForce();
					}
				} else {
					setSelectedArmy(null);
				}
			} else {
				if (clickedArmy.getForce() == currentForce()
						&& selectedArmy == null) {
					setSelectedArmy(clickedArmy);
				} else {
					if (selectedArmy != null) {
						int distance = Hexagon.getDirectDistance(selectedArmy
								.getPosition(), clickedArmy.getPosition());
						if (distance == 1) {

							if (selectedArmy.getSide() != clickedArmy.getSide()) {
								fight(selectedArmy, clickedArmy);
								if (allMoved()) {
									changeForce();
								}
							} else {

								merge(selectedArmy, clickedArmy);

								setSelectedArmy(clickedArmy);
								if (allMoved()) {
									changeForce();
								}

							}
						} else if (distance <= selectedArmy.getRemainingMove()) {
							int direction = Hexagon.getDirectDirection(
									clickedArmy.getAnimationPosition(),
									selectedArmy.getAnimationPosition());
							Point destination = Hexagon.getTranslatedCoords(
									clickedArmy.getAnimationPosition(), 1,
									direction);
							moveArm(selectedArmy, destination);
							if (allMoved()) {
								changeForce();
							}
						} else {
							if (clickedArmy.getForce() == currentForce()) {
								setSelectedArmy(clickedArmy);
							} else {
								setSelectedArmy(null);
							}
						}
					}
				}
			}
		}

	}

	/**
	 * move an army to a specified destination
	 * @param army the army
	 * @param destination the destination
	 */
	private void moveArm(Army army, Point destination) {

		Army destArm = getArmy(destination);
		if (destArm != null && destArm.getSide() == army.getSide()) {
			merge(army, destArm);
		} else {
			
			final CampaignPathFinder pathFinder=new CampaignPathFinder(army.getPosition(),destination,army,HexagonalMapModel.getInstance().getBounds());
			
			
			
			pathFinder.run();
			LinkedList path =pathFinder.getFoundPath();
			
			army.move(path,pathFinder.getPathCost());
			if (HexagonalMapModel.getInstance().isVillage(destination)
					|| HexagonalMapModel.getInstance().isCapital(destination)) {
				cityControls.remove(destination);
				cityControls.put(destination, new Integer(army.getForce()
						.getSide()));
			}
		}

	}

	//-----------property change listeners managment----------
	
	public void addPropertyChangeListener(String arg0,
			PropertyChangeListener arg1) {
		support.addPropertyChangeListener(arg0, arg1);
	}

	public void firePropertyChange(String arg0, boolean arg1, boolean arg2) {
		support.firePropertyChange(arg0, arg1, arg2);
	}

	public void firePropertyChange(String arg0, int arg1, int arg2) {
		support.firePropertyChange(arg0, arg1, arg2);
	}

	public void firePropertyChange(String arg0, Object arg1, Object arg2) {
		support.firePropertyChange(arg0, arg1, arg2);
	}

	public void removePropertyChangeListener(String arg0,
			PropertyChangeListener arg1) {
		support.removePropertyChangeListener(arg0, arg1);
	}

	public void addPropertyChangeListener(PropertyChangeListener listener) {
		support.addPropertyChangeListener(listener);
	}



	/**
	 * @return Returns the cityControls.
	 */
	public Hashtable getCityControls() {
		return cityControls;
	}

	/**
	 * @return Returns the waiting.
	 */
	public boolean isWaiting() {
		return waiting;
	}

	/**
	 * end the ccampaing
	 * @param winner the winner id
	 */
	public void endCmp(int winner) {

		String message;

		if (winner != -2) {
			String victoriousArmies = null;

			for (int i = 0; i < forces.length; i++) {
				if (forces[i].getSide() == winner) {
					if (victoriousArmies == null) {
						victoriousArmies = forces[i].getPlayerName();
					} else {
						victoriousArmies = victoriousArmies + ", "
								+ forces[i].getPlayerName();
					}

				}
			}
			message = Resources.getString("campaignVictory");
			message = message.replaceAll("<victorious side>", ""
					+ (char) ('A' + winner));
			message = message.replaceAll("<victorious players>",
					victoriousArmies);
		} else {
			message = Resources.getString("campaignStatusquo");
		}

		FB.getInstance().message(Resources.getString("endgame"));
		FB.getInstance().popUpMessage(Resources.getString("endgame"), message);

	}

	/**
	 * calculate campaign winner
	 * @return the winner id or -1 if no winner
	 */
	private int winner() {
		List remainingSides = new ArrayList();
		Integer side;
		for (int i = 0; i < forces.length; i++) {
			if (!hasLost(forces[i])) {
				side = new Integer(forces[i].getSide());
				if (!remainingSides.contains(side)) {
					remainingSides.add(side);
				}
			}
		}
		if (remainingSides.size() > 1) {
			return -1;
		} else if (remainingSides.size() == 1) {
			return ((Integer) remainingSides.get(0)).intValue();
		}
		return -2;
	}

	/**
	 * tests if specified force has lost the campaign
	 * @param force the force
	 * @return true if force has lost
	 */
	private boolean hasLost(Force force) {
		return force.getChildCount() == 0
				&& (force.getBudget() <= 12 || !controlsCapital(force.getSide()));
	}

}