/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.campaign;

import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import fb.army.hierarchy.Force;
import fb.gui.widgets.ButtonListener;
import fb.gui.widgets.FBButton;
import fb.gui.widgets.SlideValueChooser;
import fb.main.FB;
import fb.player.Player;
import fb.player.PlayersTableModel;
import fb.util.PercentageLayout;
import fb.util.Resources;
/**
 * panel where campaign game is prepared
 * @author farfpeuf
 *
 */
public class CampaignPreparationFrame extends JPanel {

	//singleton instance
	private static CampaignPreparationFrame instance;

	/**
	 * get singleton instance
	 * @return the singleton
	 */
	public static CampaignPreparationFrame getInstance() {
		if (null == instance) {
			instance = new CampaignPreparationFrame();
		}
		return instance;
	}

	/**
	 * delete singleton instance to free memory
	 *
	 */
	public static void freeInstance() {
		if (null != instance) {
			instance = null;
		}
	}

	/**
	 * build and assemble components
	 *
	 */
	public CampaignPreparationFrame() {

		FBButton fight = new FBButton(Resources.getString("fightbutton"));
		FBButton editor = new FBButton(Resources.getString("mapEditor"));
		FBButton back = new FBButton(Resources.getString("Exittomainmenu"));
		SlideValueChooser budgetChooser =
			new SlideValueChooser(Resources.getString("budget"), true);

		// cliq fight

		fight.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {

				if (testOnlyOneSide()) {
					FB.getInstance().popUpMessage(
						Resources.getString("onlyonesidetitle"),
						Resources.getString("onlyonesidetext"));
				} else {
					FB.getInstance().clearFrame();
					PlayersTableModel localPlas =
						Campaign.getInstance().getPlayersHandler().getLocalPlayersTableModel();
					Player pla;

					Force[] forces = new Force[localPlas.getNbPlayer()];
					for (int i = 0; i < forces.length; i++) {
						pla = localPlas.getPla(i);
						forces[i] =
							new Force(
								pla.getName(),
								pla.getArmyName(),
								pla.isIa(),
								pla.getSide(),
								true,
								Campaign.getInstance().getBudget());
						forces[i].setId(i);
					}
					Campaign.getInstance().setForces(forces);

					CampaignStatePanel.getInstance().buildTabs();
					FB.getInstance().dispalyCampaignGameFrame();

					CampaignGameFrame.getInstance().start();
				}
			}
		});

		editor.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				FB.getInstance().displayCampaignMapEditor();
			}
		});

		back.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				FB.getInstance().displayMainMenuFrame();
			}
		});

		budgetChooser.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				int value = ((JSlider) e.getSource()).getValue();
				Campaign.getInstance().setBudget(value);
			}
		});

		budgetChooser.setValues(500, 2000, 1000);

		setLayout(new PercentageLayout());

		add(
			CampaignPlayersPanel.getInstance(),
			new PercentageLayout.Constraint(25, 10, 50, 30));
		add(
			CampaignChooser.getInstance(true),
			new PercentageLayout.Constraint(35, 43, 30, 25));
		add(budgetChooser, new PercentageLayout.Constraint(39, 72, 22, 8));

		add(editor, new PercentageLayout.Constraint(42, 85, 15, 5));

		add(fight, new PercentageLayout.Constraint(15, 85, 15, 5));

		add(back, new PercentageLayout.Constraint(70, 85, 15, 5));

		setOpaque(false);
		validate();
		setBounds(0, 0, (int) (800 * FB.getXres()), (int) (600 * FB.getYres()));
	}

	/**
	 * test if all the players are on the same side
	 * @return true if all the players are on the same side
	 */
	public boolean testOnlyOneSide() {
		PlayersTableModel localPlas =
			Campaign.getInstance().getPlayersHandler().getLocalPlayersTableModel();
		Player pla;
		int ndPlayers = localPlas.getNbPlayer();
		int playerOneSide = localPlas.getPla(0).getSide();
		for (int i = 1; i < ndPlayers; i++) {
			pla = localPlas.getPla(i);
			if (pla.getSide() != playerOneSide) {
				return false;
			}
		}
		return true;
	}
}
