/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.campaign;


import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.io.File;

import javax.swing.JPanel;

import fb.util.FBColors;
/**
 * panel containing a small representation of the whole campaign map, used to move
 * quickly the view on some area of the campaign map
 * 
 * @author farfpeuf
 * 
 */
public class MiniHexagonalMapPanel extends JPanel {

//	 the map painting delegate
	private HexagonalMapPainter painter;
//size
	public int width, length;

	/**
	 * constructor
	 * creates painting delegate
	 */
	public MiniHexagonalMapPanel() {
		super();
		painter = new HexagonalMapPainter();
	}

	/**
	 * initialize panel
	 * @param name map name
	 * @param battleMap true if the map is  used for battle (false means map is used for campaign preparation)
	 */
	public void init(String name, boolean battleMap) {

		HexagonalMapModel.getInstance().loadFromFile(
			"campaigns" + File.separator + name);
		init(HexagonalMapModel.getInstance(), battleMap);
	}

	/**
	 * initialize panel
	 * @param model the map data model
	 * @param battleMap true if the map is  used for battle (false means map is used for campaign preparation)
	 */
	public void init(HexagonalMapModel model, boolean battleMap) {

		int MAX_WIDTH, MAX_HEIGHT;
		if (battleMap) {
			MAX_WIDTH = 144;
			MAX_HEIGHT = 96;
		} else {
			MAX_WIDTH = 192;
			MAX_HEIGHT = 128;
		}

		float ax = 1.5f * (model.getMapWidth() - 1) + 0.5f;
		float ay;
		if (model.getMapLength() % 2 == 0) {
			ay =  HexagonalMapPainter.SIN_PI_SUR_3 * 2f *(model.getMapLength() - 1);
		} else {
			ay =  HexagonalMapPainter.SIN_PI_SUR_3* ((2f * (model.getMapLength() - 1)) + 1);
		}

		float maxRadiusW = MAX_WIDTH / ax;

		float maxRadiusL = MAX_HEIGHT / ay;

		int radius;

		if (maxRadiusW > maxRadiusL) {
			radius = (int) maxRadiusL;
		} else {
			radius = (int) maxRadiusW;
		}

		width = (int) (radius * ax);
		length = (int) (radius * ay);
		//Dimension d = new Dimension(MAX_WIDTH, MAX_HEIGHT);
		Dimension d = new Dimension(width, length);
		setSize(d);
		setPreferredSize(d);
		setMinimumSize(d);
		setBackground(FBColors.background);
		painter.setModel(model);
		painter.setRadius(radius);
		repaint();
	}

	/**
	 * overridden to delegate map painting
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		painter.paintMap(g);
	}
	/**
	 * convert mouse click coordinates to hexagonal (row,column) coordinates
	 * @param mouseCoord the mouse coordinates
	 * @return the corresponding hexagonal coordinates
	 */
	public Point getHexaCoord(Point mouseCoord) {
		return painter.getHexaCoord(mouseCoord);
	}

}