/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.frames;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JPanel;

import fb.gui.panels.ChatArea;
import fb.gui.panels.LocalPlayersPanel;
import fb.gui.panels.MapChooser;
import fb.gui.panels.NetworkPlayersPanel;
import fb.gui.widgets.ButtonListener;
import fb.gui.widgets.FBButton;
import fb.gui.widgets.FBCheckBox;
import fb.main.FB;
import fb.main.Battle;
import fb.network.Server;
import fb.player.Player;
import fb.player.PlayersTableModel;
import fb.util.FBColors;
import fb.util.PercentageLayout;
import fb.util.Resources;

/**
 * frame used to host a network game
 * 
 * @author farfpeuf
 */
public class HostNetFrame extends JPanel {
	/**
	 * singleton instance
	 */
	private static HostNetFrame instance;

	/**
	 * constructor
	 * 
	 * build and assemble elements
	 * 
	 */
	private HostNetFrame() {
		super();
		// boutons etc
		FBButton startgame = new FBButton(Resources.getString("StartGame"));
		FBButton back = new FBButton(Resources.getString("Exittomainmenu"));
		FBCheckBox autoDeployCheckBox = new FBCheckBox(Resources
				.getString("autodeploy"), Battle.getInstance().isAutoDeploy());
		autoDeployCheckBox.setToolTipText(Resources.getString("deployment.tt"));
		autoDeployCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				Battle.getInstance().setAutoDeploy(
						((FBCheckBox) ev.getSource()).getValue());
			}
		});
		back.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				FB.killNet();
				FB.getInstance().displayMainMenuFrame();
			}
		});
		startgame.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				if (testOnlyOneSide()) {
					FB.getInstance().popUpMessage(
							Resources.getString("onlyonesidetitle"),
							Resources.getString("onlyonesidetext"));
				} else {
					if (Server.hasInstance()) {
						Server.getInstance().stopConnections();
					}
				}
			}
		});
		NetworkPlayersPanel netPlayersPanel = new NetworkPlayersPanel();
		JPanel allPlayers = new JPanel();
		allPlayers.setLayout(new GridLayout(2, 1));
		allPlayers.add(LocalPlayersPanel.getInstance());
		allPlayers.add(netPlayersPanel);
		allPlayers.setOpaque(false);
		setLayout(new PercentageLayout());
		JPanel chatPanel = new JPanel();
		chatPanel.setLayout(new GridLayout(1, 1));
		chatPanel
				.setBorder(BorderFactory.createLineBorder(FBColors.darkBorder));
		chatPanel.add(ChatArea.getInstance());
		add(chatPanel, new PercentageLayout.Constraint(35, 3, 30, 12));
		add(allPlayers, new PercentageLayout.Constraint(25, 17, 50, 40));
		add(MapChooser.getInstance(true), new PercentageLayout.Constraint(35, 59,
				30, 25));
		add(startgame, new PercentageLayout.Constraint(5, 90, 15, 5));
		add(autoDeployCheckBox, new PercentageLayout.Constraint(39, 86, 22, 5));
		add(back, new PercentageLayout.Constraint(80, 90, 15, 5));
		setOpaque(false);
		validate();
		setBounds(0, 0, (int) (800 * FB.getXres()), (int) (600 * FB.getYres()));

	}

	/**
	 * get singleton instance
	 * 
	 * @return the singleton
	 */
	public static synchronized HostNetFrame getInstance() {
		if (null == instance) {
			instance = new HostNetFrame();
		}
		return instance;
	}

	/**
	 * delete singleton to free memory
	 * 
	 */
	public static synchronized void freeInstance() {
		if (null != instance) {
			instance = null;
		}
	}

	/**
	 * test if all armies are on the same side
	 * 
	 * @return true if all armies aare on the same side
	 */
	public boolean testOnlyOneSide() {
		PlayersTableModel localPlas = Battle.getInstance().getPlayersHandler()
				.getLocalPlayersTableModel();
		Player pla;
		int ndPlayers = localPlas.getNbPlayer();
		int playerOneSide = localPlas.getPla(0).getSide();
		for (int i = 1; i < ndPlayers; i++) {
			pla = localPlas.getPla(i);
			if (pla.getSide() != playerOneSide) {
				return false;
			}
		}
		PlayersTableModel netPlas = Battle.getInstance().getPlayersHandler()
				.getNetPlayersTableModel();
		ndPlayers = netPlas.getNbPlayer();
		for (int i = 0; i < ndPlayers; i++) {
			pla = netPlas.getPla(i);
			if (pla.getSide() != playerOneSide) {
				return false;
			}
		}
		return true;
	}
}