/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.frames;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JPanel;

import fb.gui.panels.ChatArea;
import fb.gui.panels.LocalPlayersPanel;
import fb.gui.panels.MapChooser;
import fb.gui.panels.NetworkPlayersPanel;
import fb.gui.widgets.ButtonListener;
import fb.gui.widgets.FBButton;
import fb.gui.widgets.FBCheckBox;
import fb.main.FB;
import fb.main.Battle;
import fb.network.Client;
import fb.util.FBColors;
import fb.util.PercentageLayout;
import fb.util.Resources;

/**
 * frame used to join a network game
 * @author farfpeuf
 */
public class JoinNetFrame extends JPanel {

	/**
	 * singleton instance
	 */
	private static JoinNetFrame instance;

	/**
	 * get singleton instance
	 * @return the singleton
	 */
	public static  synchronized JoinNetFrame getInstance() {
		if(null==instance){
			instance=new JoinNetFrame();
		}
		return instance;
	}
    /**
     * delete singleton to free memory
     *
     */
    public static  synchronized void freeInstance() {
        if (null != instance) {
            instance = null;
        }
    }
	/**
	 * constructor
	 * 
	 * build and assemble elements
	 * 
	 */
	private JoinNetFrame() {
		super();

		//boutons etc

		FBButton notify = new FBButton(Resources.getString("Ready"));

		FBButton back = new FBButton(Resources.getString("Exittomainmenu"));
		FBCheckBox autoDeployCheckBox=new FBCheckBox(Resources.getString("autodeploy"),Battle.getInstance().isAutoDeploy());
		autoDeployCheckBox.setToolTipText(Resources.getString("deployment.tt"));
        autoDeployCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				Battle.getInstance().setAutoDeploy(((FBCheckBox)ev.getSource()).getValue());
			}
		});		

		NetworkPlayersPanel netPlayersPanel = new NetworkPlayersPanel();

		back.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				FB.killNet();
				FB.getInstance().displayMainMenuFrame();
			}
		});

		notify.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				if (FB.isNet()) {
					Client.getInstance().send("READY" + Client.getInstance().getClientName());
				}
			}
		});

		JPanel panel = new JPanel();
		panel.setLayout(new GridLayout(2,1));

		panel.add(LocalPlayersPanel.getInstance());
		panel.add(netPlayersPanel);


		panel.setOpaque(false);
		panel.validate();

		panel.setBounds(
			(int) ((800 * FB.getXres() - 500) / 2),
			(int) ((600 * FB.getYres() - 500) / 2),
			500,
			500);

		setLayout(new PercentageLayout());
		

		JPanel cadrePanel = new JPanel();
		cadrePanel.setLayout(new GridLayout(1,1));
		cadrePanel.setBorder(BorderFactory.createLineBorder(FBColors.darkBorder));
		cadrePanel.add(ChatArea.getInstance());
		add(
				cadrePanel,
					new PercentageLayout.Constraint(35, 3, 30, 12));		
		add(panel, new PercentageLayout.Constraint(25, 17, 50, 40));
		add(MapChooser.getInstance(false), new PercentageLayout.Constraint(35, 59, 30, 25));
		add(autoDeployCheckBox, new PercentageLayout.Constraint(39, 86, 22, 5));

		add(notify, new PercentageLayout.Constraint(5, 90, 15, 5));

		add(back, new PercentageLayout.Constraint(80, 90, 15, 5));		
		
		setOpaque(false);
		setSize((int) (800 * FB.getXres()), (int) (600 * FB.getYres()));

	}

}
