/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.frames;
import java.awt.GridLayout;
import java.io.File;

import javax.swing.JPanel;

import fb.gui.widgets.ButtonListener;
import fb.gui.widgets.FBButton;
import fb.main.FB;
import fb.util.Resources;
/**
 * Fantasy Battlefields main menu frame
 * @author Farfpeuf.
 */
public class MainMenuFrame extends JPanel {
	/**
	 * singleton instance
	 */
	private static MainMenuFrame instance;
	/**
	 * buttons
	 */
	private FBButton openArmyEditor,
		play,manual,
		loadGame,
		playcampagne,
		playlan,
		options,
		about,
		exit;
	/**
	 * get singleton instance
	 * @return the singleton
	 */
	public static  synchronized MainMenuFrame getInstance() {
		if (null == instance) {
			instance = new MainMenuFrame();
		}
		return instance;
	}
	/**
	 * delete singlton instance to free memory
	 *
	 */
	public static synchronized  void freeInstance() {
		if (null != instance) {
			instance = null;
		}
	}
	/**
	 * handle buttons enabling
	 *
	 */
	public void enableButtons() {
		playcampagne.setEnabled(true);
		File gamesdir = new File("games");
		if (!gamesdir.exists()) {
			gamesdir.mkdir();
		}
		String[] filelist = gamesdir.list();
		loadGame.setEnabled(filelist.length != 0);
		File armiesdir = new File("armies");
		if (!armiesdir.exists()) {
			armiesdir.mkdir();
		}
		filelist = armiesdir.list();
		boolean atLeastOneArmy = filelist.length != 0;
		play.setEnabled(atLeastOneArmy);
		playlan.setEnabled(atLeastOneArmy);
	}
	/**
	 * constructor
	 * 
	 * build and assemble elements
	 * 
	 */
	public MainMenuFrame() {
		openArmyEditor = new FBButton(Resources.getString("armyEditor"));
		play = new FBButton(Resources.getString("playquick"));
		manual= new FBButton(Resources.getString("gameManual"));
		loadGame = new FBButton(Resources.getString("loadGame"));
		playcampagne = new FBButton(Resources.getString("playcamp"));
		playlan = new FBButton(Resources.getString("playnet"));
		options = new FBButton(Resources.getString("options"));
		about = new FBButton(Resources.getString("credits"));
		exit = new FBButton(Resources.getString("exit"));
		loadGame.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				FB.getInstance().loadGameAction();
			}
		});
		openArmyEditor.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				FB.getInstance().openEditor();
			}
		});

		play.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				FB.getInstance().displayLocalFrame();
			}
		});
		playcampagne.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				FB.getInstance().displayCampaignPreparationFrame();
			}
		});
		playlan.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				FB.getInstance().displayLanFrame();
			}
		});
		manual.addActionListener(new ButtonListener() {
					public void threadedActionPerformed() {
						FB.getInstance().displayManualFrame();
					}
				});
		options.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				FB.getInstance().displayOptionFrame();
			}
		});
		about.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				String message=FB.getAboutMessage().replaceAll("\\n","<BR>");
				FB.getInstance().popUpMessage(Resources.getString("credits"),message);
			}
		});
		exit.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				FB.safeExit();
			}
		});
		setLayout(new GridLayout(10, 1));
		add(openArmyEditor);
		add(play);
		add(loadGame);
		add(playlan);
		add(playcampagne);
		add(manual);
		add(options);
		add(about);
		add(exit);
		validate();
		setOpaque(false);
		setBounds(
			(int) (300 * FB.getXres()),
			(int) (200 * FB.getYres()),
			(int) (200 * FB.getXres()),
			(int) (200 * FB.getYres()));
	}
}