/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.frames;

import java.awt.DisplayMode;
import java.awt.GridLayout;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.JPanel;
import javax.swing.ToolTipManager;

import fb.gui.widgets.ButtonListener;
import fb.gui.widgets.DisplayModeCell;
import fb.gui.widgets.FBButton;
import fb.gui.widgets.FBCheckBox;
import fb.gui.widgets.FBComboBox;
import fb.main.FB;
import fb.main.Options;
import fb.sound.MusicHandler;
import fb.util.GameInstaller;
import fb.util.Resources;

/**
 * frame used to edit Fantasy Battlefields options
 * 
 * @author Farfpeuf
 */
public class OptionFrame extends JPanel {
	/**
	 * singleton instance
	 */
	private static OptionFrame instance;

	/**
	 * get singleton instance
	 * 
	 * @return the singleton
	 */
	public static  synchronized OptionFrame getInstance() {
		if (null == instance) {
			instance = new OptionFrame();
		}
		return instance;
	}

	/**
	 * delete singleton to free memory
	 * 
	 */
	public static  synchronized void freeInstance() {
		if (null != instance) {
			instance = null;
		}
	}

	/**
	 * combo boxes
	 */
	private FBComboBox langchooser, displayModeChooser;

	/**
	 * check boxes
	 */
	private FBCheckBox music, sound, tooltips;

	/**
	 * constructor
	 * 
	 * build and assemble elements
	 * 
	 */
	public OptionFrame() {

		music = new FBCheckBox(Resources.getString("Music"), Options
				.isMusicOn());

		tooltips = new FBCheckBox(Resources.getString("enabletootips"), Options
				.isTooltipsEnabled());

		// music.setEnabled(false);

		sound = new FBCheckBox(Resources.getString("Sound"), Options
				.isSoundOn());
		FBButton save, back;
		back = new FBButton(Resources.getString("Cancel"));

		save = new FBButton(Resources.getString("Ok"));

		langchooser = new FBComboBox(GameInstaller.getAvailableLanguages());
		langchooser.setSelectedItem(GameInstaller.getLocaleName(new Locale(
				Options.getLanguage(), Options.getCountry())));

		DisplayMode[] availableModes = FB.getGraphicsDevice().getDisplayModes();
		Vector modes = new Vector();
		DisplayMode tmpMode, currentMode = null;
		for (int i = 0; i < availableModes.length; i++) {
			tmpMode = availableModes[i];
			if (tmpMode.equals(FB.getGraphicsDevice().getDisplayMode())) {
				currentMode = tmpMode;
			}
			if (tmpMode.getBitDepth() >= 16 && tmpMode.getWidth() >= 1024
					&& tmpMode.getHeight() * 4 / 3 == tmpMode.getWidth()) {
				modes.add(tmpMode);
			}
		}
		displayModeChooser = new FBComboBox(modes);
		displayModeChooser.setRenderer(new DisplayModeCell());
		displayModeChooser.setSelectedItem(currentMode);
		displayModeChooser.setEnabled(false); // TODO

		save.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {

				Options.setMusicOn(music.value);
				Options.setSoundOn(sound.value);
				Options.setTooltipsEnabled(tooltips.value);

				Options.setLanguage(((String) langchooser.getSelectedItem())
						.toLowerCase().substring(0, 2));
				DisplayMode mode = (DisplayMode) displayModeChooser
						.getSelectedItem();

				Options.setDisplayMode(mode);
				if (mode != null) {
					FB.setDisplayMode(mode);
				}

				if (Options.getLanguage().equals("fr")) {
					Options.setCountry("FR");
				} else if (Options.getLanguage().equals("it")) {
					Options.setCountry("IT");
				} else {
					Options.setCountry("UK");
				}
				Resources.setResourceBundle(ResourceBundle.getBundle(
						"Messages", new Locale(Options.getLanguage(), Options
								.getCountry())));
				if (Options.isMusicOn()) {
					MusicHandler.playRandomTrack();
				} else {
					MusicHandler.stop();
				}
				Options.getInstance().save();
				ToolTipManager.sharedInstance().setEnabled(
						Options.isTooltipsEnabled());
				FB.getInstance().displayMainMenuFrame();
			}
		});

		back.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				FB.getInstance().displayMainMenuFrame();
			}
		});

		;
		setLayout(new GridLayout(7, 1));
		add(music);
		add(sound);
		add(tooltips);
		add(langchooser);
		add(displayModeChooser);
		add(save);
		add(back);
		setOpaque(false);
		validate();
		setBounds((int) ((800 * FB.getXres() - 200) / 2), (int) ((600 * FB
				.getYres() - 200) / 2), 200, 200);
	}

}
