/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.widgets;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * a widget to choose a numerical value
 * 
 * @author farfpeuf
 * 
 */
public class ButtonValueChooser extends JPanel {
	/**
	 * label displaying current value
	 */
	private JLabel valuefield;

	/**
	 * buttons to increase or decrease value
	 */
	private FBButton plus, minus;

	/**
	 * current minimun and maximum value
	 */
	private int value, min, max;

	/**
	 * constructor build and assemble components
	 * 
	 */
	public ButtonValueChooser() {
		setLayout(new GridBagLayout());
		valuefield = new JLabel();
		valuefield.setHorizontalAlignment(0);
		plus = new FBButton("+", null, true);
		minus = new FBButton("-", null, true);
		plus.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				if (value < max) {
					value++;
					minus.setEnabled(true);
					if (value == max) {
						plus.setEnabled(false);
					}
					valuefield.setText(Integer.toString(value));
				}
			}
		});
		minus.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				if (value > min) {
					value--;
					plus.setEnabled(true);
					if (value == min) {
						minus.setEnabled(false);
					}
					valuefield.setText(Integer.toString(value));
				}
			}
		});
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.weightx = 1;
		gbc.weighty = 1;
		gbc.fill = GridBagConstraints.BOTH;
		add(plus, gbc);
		gbc.gridy++;
		add(minus, gbc);
		gbc.gridy--;
		gbc.gridx++;
		gbc.gridheight = 2;
		add(valuefield, gbc);
		setSize(60, 60);
		validate();
	}

	/**
	 * set current minimun and maximum value
	 * 
	 * @param mn
	 *            the nminimum value
	 * @param mx
	 *            the maximum valur
	 * @param init
	 *            the initial value
	 */
	public void setValues(int mn, int mx, int init) {
		min = mn;
		max = mx;
		value = init;
		if (init == min) {
			minus.setEnabled(false);
		}
		if (init == max) {
			plus.setEnabled(false);
		}
		valuefield.setText(Integer.toString(init));
	}

	/**
	 * @return Returns the current value.
	 */
	public int getValue() {
		return value;
	}
}