/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.widgets;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.ImageIcon;
import javax.swing.JComponent;

import fb.util.FBColors;
import fb.util.Resources;

/**
 * a fantasy battlefields custom check box
 * 
 * @author farfpeuf
 * 
 */
public class FBCheckBox extends JComponent implements MouseListener {
	/**
	 * box icons
	 */
	public ImageIcon courant, checked, unchecked, pressed, disabled,
			undisabled;

	/**
	 * box current value
	 */
	public boolean value = false;

	/**
	 * box label
	 */
	protected String label;

	/**
	 * box action listener
	 */
	protected ActionListener actionlistener = null;

	/**
	 * is box enabled ?
	 */
	protected boolean enabled = true;

	/**
	 * is mouse currenlty over the box
	 */
	protected boolean mouseOver = false;

	/**
	 * is box opaque
	 */
	protected boolean opaque = true;

	/**
	 * the box text color
	 */
	private Color textcolor = FBColors.buttonText;

	/**
	 * constructor
	 * 
	 * @param txt
	 *            the checkbox label
	 */
	public FBCheckBox(String txt) {
		this(txt, false);
	}

	/**
	 * constructor
	 * 
	 * @param txt
	 *            the checkbox label
	 * @param init
	 *            the initial value
	 */
	public FBCheckBox(String txt, boolean init) {
		super();
		value = init;
		label = txt;
		checked = Resources.getLookNFeelImage("checkedbox");
		unchecked = Resources.getLookNFeelImage("uncheckedbox");
		pressed = Resources.getLookNFeelImage("pressedbox");
		disabled = Resources.getLookNFeelImage("disabledbox");
		undisabled = Resources.getLookNFeelImage("undisabledbox");
		label = txt;
		if (value) {
			setIcon(checked);
		} else {
			setIcon(unchecked);
		}
		addMouseListener(this);

	}

	/**
	 * change box icon
	 * 
	 * @param icon
	 *            the new icon
	 */
	private void setIcon(ImageIcon icon) {
		courant = icon;
		repaint();
	}

	// no border
	public void paintBorder(Graphics g) {

	}

	/**
	 * paint the box
	 */
	public void paintComponent(Graphics g) {
		g.drawImage(courant.getImage(), 0, 0, getWidth(), getHeight(), null);

		if (label != null) {
			g.setColor(textcolor);
			java.awt.font.FontRenderContext frc = ((Graphics2D) g)
					.getFontRenderContext();
			java.awt.geom.Rectangle2D r2d = g.getFont().getStringBounds(label,
					frc);

			int x = (int) ((getWidth() - r2d.getWidth()) / 2);
			int y = (int) ((getHeight() + r2d.getHeight()) / 2);

			g.drawString(label, x, y);
		}
	}

	// mouse listener implementation
	public void mouseClicked(MouseEvent event) {
	}

	/**
	 * box pressed
	 */
	public void mousePressed(MouseEvent event) {
		if (enabled) {
			setIcon(pressed);
		}
	}

	/**
	 * box released
	 */
	public void mouseReleased(MouseEvent event) {
		if (enabled) {
			clic();
		}
	}

	/**
	 * set box action listener
	 * 
	 * @param al
	 *            the action listener
	 */
	public void addActionListener(ActionListener al) {
		actionlistener = al;
	}

	/**
	 * change box value
	 * 
	 * @param selected
	 *            the new value
	 */
	public void setSelected(boolean selected) {
		value = selected;
		if (value) {
			setIcon(checked);
		} else {
			setIcon(unchecked);
		}

	}

	/**
	 * box clicked
	 * 
	 */
	protected void clic() {
		setSelected(!value);
		if (actionlistener != null) {
			actionlistener.actionPerformed(new ActionEvent(this, 0, "clic"));
		}
	}

	/**
	 * enable or disable the box
	 */
	public void setEnabled(boolean enabled) {
		this.enabled = enabled;
		if (!enabled) {
			if (value) {
				setIcon(disabled);
			} else {
				setIcon(undisabled);
			}
			textcolor = Color.lightGray;
		} else {
			if (value) {
				setIcon(checked);
			} else {
				setIcon(unchecked);
			}
			textcolor = FBColors.buttonText;
		}
	}

	/**
	 * @return Returns the label.
	 */
	public String getText() {
		return label;
	}

	/**
	 * @param label
	 *            The label to set.
	 */
	public void setText(String label) {
		this.label = label;
	}

	/**
	 * @return Returns the value.
	 */
	public boolean getValue() {
		return value;
	}

	/**
	 * @param value
	 *            The value to set.
	 */
	public void setValue(boolean value) {
		this.value = value;
	}

	/**
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent e) {

	}

	/**
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent e) {

	}
}
