/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.widgets.armytreetable;

import java.awt.Component;

import javax.swing.JLabel;
import javax.swing.JTree;
import javax.swing.tree.TreeCellRenderer;

import fb.army.entities.Entity;
import fb.army.entities.Unit;
import fb.army.hierarchy.HierarchyNode;
import fb.util.FBColors;
import fb.util.Resources;

/**
 * an renderer for army nodes in army tree-table
 * 
 * @author Farfpeuf
 */
public class ArmyNodeTreeCellRenderer extends JLabel implements
		TreeCellRenderer {

	/**
	 * tree-table used in army editor or in battle information table ?
	 */
	private boolean battleMode;
	
	/**
	 * Tree Cell Renderer implementation
	 */

	public Component getTreeCellRendererComponent(JTree tree, Object value,
			boolean sel, boolean expanded, boolean leaf, int row,
			boolean hasFocus) {
		String stringValue = getNodeString(value);
		if (null == stringValue) {
			stringValue = tree.convertValueToText(value, sel, expanded, leaf,
					row, hasFocus);
		}
		// this.tree = tree;
		setText(stringValue);
		setBackground(FBColors.background);
		setIcon(((HierarchyNode) value).getMinicon());
		setDisabledIcon(((HierarchyNode) value).getMinicon());
		if (sel)
			setForeground(FBColors.highlightedText);
		else
			setForeground(FBColors.standardText);
		// There needs to be a way to specify disabled icons.
		if (!tree.isEnabled()) {
			setEnabled(false);
		} else {
			setEnabled(true);
		}
		setComponentOrientation(tree.getComponentOrientation());
		setToolTipText(getToolTip(value));
		return this;
	}

	/**
	 * get node tooltip
	 * 
	 * @param value
	 *            node value
	 * @return the tooltip
	 */
	public String getToolTip(Object value) {
		if (value instanceof Entity) {
			String tooltip = ((Entity) value).getTooltip();
			if (!tooltip.startsWith("<html>")) {
				tooltip = "<html>" + tooltip;
			} else {
				tooltip = tooltip.substring(0, tooltip.length() - 7);
			}
			if (battleMode) {
				Entity e = (Entity) value;
				if (e.getState() == 'a') {
					tooltip += Resources.getString("annihilatedimage.tt");
				} else if (e.getState() == 'r') {
					tooltip += Resources.getString("routedimage.tt");
				} else if (e.getState() == 's') {
					tooltip += Resources.getString("shakenimage.tt");
				} else if (e.getState() == 'b') {
					tooltip += Resources.getString("bloodlustimage.tt");
				}

			}
			if (value instanceof Unit) {
				Unit u = (Unit) value;

				if (battleMode) {
					char order = u.getOrder();
					if (order == 'a') {
						tooltip += Resources.getString("chargeimage.tt");
					} else if (order == 'o') {
						tooltip += Resources.getString("manoeuvreimage.tt");
					} else if (order == 'h') {
						tooltip += Resources.getString("defendimage.tt");
					}
				}

				if (battleMode && u.isDisorganized()) {
					tooltip += Resources.getString("disorganizedimage.tt");
				}
				if (u.hasChief()) {
					tooltip += Resources.getString("chiefimage.tt");
				}
				if (u.hasChampion()) {
					tooltip += Resources.getString("championimage.tt");
				}
				if (u.hasStandardBearer()) {
					tooltip += Resources.getString("standardimage.tt");
				}
				if (u.getMusicians() > 0) {
					tooltip += Resources.getString("musicianimage.tt");
				}
				if (u.isUpgraded()) {
					tooltip += Resources.getString("upgradedimage.tt");
				}

			}

			tooltip += "</html>";
			return tooltip;

		} else {
			return null;
		}
	}

	/**
	 * get node string value
	 * 
	 * @param value
	 *            node object value
	 * @return the node string value
	 */
	private String getNodeString(Object value) {
		if (value instanceof HierarchyNode) {
			HierarchyNode a = (HierarchyNode) value;
			return a.getFullName();
		} else {
			return null;
		}
	}

	/**
	 * constructor
	 * 
	 * @param battleMode
	 *            tree-table used in army editor or in battle information table ?
	 */
	public ArmyNodeTreeCellRenderer(boolean battleMode) {
		super();
		this.battleMode = battleMode;
	}


}