/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.main;

import java.io.BufferedReader;
import java.io.File;
import java.io.Reader;
import java.util.Vector;

import fb.army.entities.Entity;
import fb.util.EntityFactory;
import fb.util.FileTools;

/**
 * catalog of all available entities
 * 
 * @author farfpeuf
 * 
 */
public class Catalog {
	/**
	 * entities by race
	 */
	private static Vector undeadlist = new Vector(), amazonlist = new Vector(),
			barbarianlist = new Vector(), mercenarylist = new Vector(),
			hugelist = new Vector(), trolllist = new Vector(),
			ogrelist = new Vector(), goblinlist = new Vector(),
			dwarflist = new Vector(), woodelflist = new Vector(),
			humanlist = new Vector(), orclist = new Vector(),
			highelflist = new Vector(), daemonlist = new Vector(),
			allList = null;

	/**
	 * @return the full catalog
	 */
	public static synchronized Vector getFullCatalog() {
		if (allList == null) {
			allList = createcatalog("lists" + File.separator + "catalog.txt");
		}
		return allList;
	}

	/**
	 * get available entities by race
	 * 
	 * @param fblstname
	 *            the race
	 * @return available entities for specified race
	 */
	public static Vector getAvailableEntities(String fblstname) {
		getFullCatalog();

		Vector availableEntities = null;
		if (fblstname.equals("undead")) {
			availableEntities = undeadlist;
		} else if (fblstname.equals("amazon")) {
			availableEntities = amazonlist;
		} else if (fblstname.equals("barbarian")) {
			availableEntities = barbarianlist;
		} else if (fblstname.equals("mercenary")) {
			availableEntities = mercenarylist;
		} else if (fblstname.equals("hugecreature")) {
			availableEntities = hugelist;
		} else if (fblstname.equals("ogre")) {
			availableEntities = ogrelist;
		} else if (fblstname.equals("troll")) {
			availableEntities = trolllist;
		} else if (fblstname.equals("goblin")) {
			availableEntities = goblinlist;
		} else if (fblstname.equals("dwarf")) {
			availableEntities = dwarflist;
		} else if (fblstname.equals("woodelf")) {
			availableEntities = woodelflist;
		} else if (fblstname.equals("human")) {
			availableEntities = humanlist;
		} else if (fblstname.equals("orc")) {
			availableEntities = orclist;
		} else if (fblstname.equals("high-elf")) {
			availableEntities = highelflist;
		} else if (fblstname.equals("daemon")) {
			availableEntities = daemonlist;
		} else {
			FB.debug("no list for " + fblstname);
			availableEntities = barbarianlist;
		}
		return availableEntities;

	}

	/**
	 * create the catalog
	 * 
	 * @param path
	 *            catalog path
	 * @return the catalog
	 */
	private static Vector createcatalog(String path) {
		Vector catalog = new Vector();
		Reader input_file = null;

		String ligne = "";
		Entity entity;
		BufferedReader br;

		try {
			input_file = FileTools.getReader(path);
			br = new BufferedReader(input_file);
			int linenumber = 0;
			while (ligne != null) {
				ligne = br.readLine();
				if (ligne != null) {
					entity = EntityFactory.buildEntity(ligne, linenumber);
					if (entity != null) {

						entity.internationalize();
						catalog.add(entity);
						linenumber++;
						String race = entity.getRace();

						if (race.equals("undead")) {
							undeadlist.add(entity);
						} else if (race.equals("amazon")) {
							amazonlist.add(entity);
						} else if (race.equals("barbarian")) {
							barbarianlist.add(entity);
						} else if (race.equals("half-orc")) {
							mercenarylist.add(entity);
						} else if (race.equals("horse-raider")) {
							mercenarylist.add(entity);
						} else if (race.equals("giant")) {
							hugelist.add(entity);
						} else if (race.equals("dragon")) {
							hugelist.add(entity);
						} else if (race.equals("troll")) {
							trolllist.add(entity);
						} else if (race.equals("ogre")) {
							ogrelist.add(entity);
						} else if (race.equals("goblin")) {
							goblinlist.add(entity);
						} else if (race.equals("dwarf")) {
							dwarflist.add(entity);
						} else if (race.equals("woodelf")) {
							woodelflist.add(entity);
						} else if (race.equals("human")) {
							humanlist.add(entity);
						} else if (race.equals("orc")) {
							orclist.add(entity);
						} else if (race.equals("high-elf")) {
							highelflist.add(entity);
						} else if (race.equals("daemon")) {
							daemonlist.add(entity);
						}
					}
				} // if
			} // while
			br.close();
		} // try
		catch (Exception e) {
			e.printStackTrace();
		}
		return catalog;
	}

}
