/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.network;

import java.util.StringTokenizer;

import fb.army.entities.Entity;
import fb.army.entities.FlyingEntity;
import fb.army.entities.SpellCaster;
import fb.army.entities.SummonedCreature;
import fb.army.entities.Unit;
import fb.army.hierarchy.Army;
import fb.army.hierarchy.Command;
import fb.army.hierarchy.MercenaryCommand;
import fb.battlefield.BattleFieldModel;
import fb.gui.panels.ChatArea;
import fb.gui.panels.LocalPlayersPanel;
import fb.gui.panels.MapChooser;
import fb.main.FB;
import fb.main.Battle;
import fb.phases.BinaryPhase;
import fb.phases.MovementPhase;
import fb.phases.NetSpellInterruption;
import fb.phases.SynchronizedPhase;
import fb.spells.BannishDaemon;
import fb.spells.BasicSpell;
import fb.spells.Lightnings;
import fb.spells.EnergyBall;
import fb.spells.Heal;
import fb.spells.MagicShield;
import fb.spells.ProjectileShield;
import fb.spells.Resurection;
import fb.spells.Shrink;
import fb.spells.Speed;
import fb.spells.SummonDaemon;
import fb.spells.Sunrise;
import fb.spells.TargetSpell;
import fb.spells.Terror;
import fb.spells.Telepathy;
import fb.spells.ValuedSpell;
import fb.util.FBColors;
import fb.util.NetworkDX;
import fb.util.Resources;

/**
 * network messages interpreter
 * @author Farfpeuf
 */
public class MessagesInterpreter implements MessagesConstants {
	
	/**
	 * interprete network message
	 * @param str the message
	 */
	public static void interprete(String str) {
		if (str.startsWith(INITGAME)) {
			Army[] armies = new Army[Battle.getInstance().getPlayersHandler()
									.getTotalNbPlayers()];
			Battle.getInstance().setArmies(armies);
			int index1 = str.indexOf('#');
			int nbClient = Integer.parseInt(str.substring(INITGAME.length(),
					index1));
			long netDXSeed = Long.parseLong(str.substring(index1 + 1));
			NetworkDX.initRNG(netDXSeed);
			Client.getInstance().setNbClients(nbClient);
			Client.getInstance().sendArmies();
		} else if (str.startsWith(SYNCHRONIZE)) {
			Client.getInstance().notifySynchronize(str);
		} else if (str.startsWith(MAGICSYNCHRONIZE)) {
			Client.getInstance().notifyMagicSynchronize();
		} else if (str.startsWith(ROLL)) {
			int roll = Integer.parseInt(str.substring(ROLL.length()));
			Client.getInstance().notifyRoll(roll);
		} else if (str.startsWith(ENDGAME)) {
			int winner = Integer.parseInt(str.substring(ENDGAME.length()));
			Battle.getLoop().doEndBattle(winner);
		} else if (str.startsWith(MYARMIESARE)) {
			int index1, index2, index3, index4;
			int i = 0;
			index1 = str.indexOf('#');
			int clientId = Integer.parseInt(str.substring(MYARMIESARE.length(),
					index1));
			String armiesList = str.substring(index1 + 1);
			Army[] armies=Battle.getInstance().getArmies();
			while (!armiesList.equals("")) {
				index1 = armiesList.indexOf('#');
				index2 = index1 + 1
						+ armiesList.substring(index1 + 1).indexOf('#');
				index3 = index2 + 1
						+ armiesList.substring(index2 + 1).indexOf('#');
				index4 = index3 + 1
						+ armiesList.substring(index3 + 1).indexOf('#');
				String playerName = armiesList.substring(0, index1);
				boolean playerIA = Boolean.valueOf(
						armiesList.substring(index1 + 1, index2))
						.booleanValue();
				int playerSide = Integer.parseInt(armiesList.substring(
						index2 + 1, index3));
				String filestream = armiesList.substring(index3 + 1, index4);
				armiesList = armiesList.substring(index4 + 1);
				filestream = filestream.replaceAll("@", "\n");
				boolean found = false;
				while (i < armies.length && !found) {
					if (armies[i] == null) {
						armies[i] = new Army(playerName, filestream,
								playerIA, playerSide, Client.getInstance()
										.getClientId() == clientId);
						armies[i].setGmId(i);
						found = true;
					}
					i++;
				}
			}

			if (i == armies.length) {
				FB.getInstance().playGame(armies,false,false);
			}
		} else if (str.startsWith(MOVETO)) {
			int index1 = str.indexOf('#');
			int index2 = str.lastIndexOf('#');
			int entityId = Integer.parseInt(str.substring(MOVETO.length(),
					index1));
			int x = Integer.parseInt(str.substring(index1 + 1, index2));
			int y = Integer.parseInt(str.substring(index2 + 1));
			((Entity) Battle.getInstance().getAllEntities().get(entityId))
					.doMoveTo(x, y);
		} else if (str.startsWith(ACTORNTARGET)) {
			int index = str.indexOf('#');
			int actorId = Integer.parseInt(str.substring(ACTORNTARGET.length(),
					index));
			int targetId = Integer.parseInt(str.substring(index + 1));
			BinaryPhase currentphase = (BinaryPhase) Battle.getLoop().currentSynchronizedPhase;
			currentphase.doAddActorAndTarget(((Entity) Battle.getInstance()
					.getAllEntities().get(actorId)), ((Entity) Battle
					.getInstance().getAllEntities().get(targetId)));
		} else if (str.startsWith(WOUNDMODEL)) {
			int index = str.indexOf('#');
			int modelId = Integer.parseInt(str.substring(WOUNDMODEL.length(),
					index));
			int wounds = Integer.parseInt(str.substring(index + 1));
			if (modelId != -1) {
				BattleFieldModel.getInstance().getWarrior(modelId).doWound(wounds);
			}
		} else if (str.startsWith(SETMORALE)) {
			int index = str.indexOf('#');
			int entityId = Integer.parseInt(str.substring(SETMORALE.length(),
					index));
			int score = Integer.parseInt(str.substring(index + 1));
			((Unit) Battle.getInstance().getAllEntities().get(entityId))
					.doSetMorale(score);
		} else if (str.startsWith(DONE)) {
			int armyID = Integer.parseInt(str.substring(DONE.length()));
			if (Battle.getLoop() != null) {
				Battle.getLoop().doSetToken(armyID);
			}
		} else if (str.startsWith(ENDTURN)) {
			int armyID = Integer.parseInt(str.substring(ENDTURN.length()));
			SynchronizedPhase currentphase = Battle.getLoop()
					.getCurrentSynchronizedPhase();
			if (currentphase instanceof MovementPhase) {
				((MovementPhase) currentphase).endOfTurn(armyID);
			}
		} else if (str.startsWith(GROUPREACTION)) {
			int index1 = str.indexOf('#');
			int index2 = str.lastIndexOf('#');
			int armyId = Integer.parseInt(str.substring(GROUPREACTION.length(),
					index1));
			int groupId = Integer.parseInt(str.substring(index1 + 1, index2));
			int score = Integer.parseInt(str.substring(index2 + 1));
			((Command) Battle.getInstance().getArmies()[armyId].getCommands().get(groupId))
					.doGroupReaction(score);
		} else if (str.startsWith(EXPEND)) {
			int index = str.indexOf('#');
			int entityId = Integer.parseInt(str.substring(EXPEND.length(),
					index));
			int plus = Integer.parseInt(str.substring(index + 1));
			((Unit) Battle.getInstance().getAllEntities().get(entityId))
					.doExpend(plus);
		} else if (str.startsWith(REDUCE)) {
			int index = str.indexOf('#');
			int entityId = Integer.parseInt(str.substring(REDUCE.length(),
					index));
			int minus = Integer.parseInt(str.substring(index + 1));
			((Unit) Battle.getInstance().getAllEntities().get(entityId))
					.doReduce(minus);
		} else if (str.startsWith(REORGANIZE)) {
			int entityId = Integer.parseInt(str.substring(REORGANIZE.length()));
			((Unit) Battle.getInstance().getAllEntities().get(entityId))
					.doReorganize();
		} else if (str.startsWith(LEFT90)) {
			int entityId = Integer.parseInt(str.substring(LEFT90.length()));
			((Unit) Battle.getInstance().getAllEntities().get(entityId))
					.doLeft90();
		} else if (str.startsWith(RIGHT90)) {
			int entityId = Integer.parseInt(str.substring(RIGHT90.length()));
			((Unit) Battle.getInstance().getAllEntities().get(entityId))
					.doRight90();
		} else if (str.startsWith(SURROUND)) {
			int entityId = Integer.parseInt(str.substring(SURROUND.length()));
			((Unit) Battle.getInstance().getAllEntities().get(entityId))
					.doSurround();
		} else if (str.startsWith(LEFTWHEEL)) {
			int index = str.indexOf('#');
			int entityId = Integer.parseInt(str.substring(LEFTWHEEL.length(),
					index));
			double theta = Float.parseFloat(str.substring(index + 1));
			((Unit) Battle.getInstance().getAllEntities().get(entityId))
					.doLeftWheel(theta);
		} else if (str.startsWith(RIGHTWHEEL)) {
			int index = str.indexOf('#');
			int entityId = Integer.parseInt(str.substring(RIGHTWHEEL.length(),
					index));
			double theta = Float.parseFloat(str.substring(index + 1));
			((Unit) Battle.getInstance().getAllEntities().get(entityId))
					.doRightWheel(theta);
		} else if (str.startsWith(ROTATE)) {
			int index = str.indexOf('#');
			int entityId = Integer.parseInt(str.substring(ROTATE.length(),
					index));
			double theta = Float.parseFloat(str.substring(index + 1));
			((Entity) Battle.getInstance().getAllEntities().get(entityId))
					.doRotate(theta);
		} else if (str.startsWith(TAKEOFF)) {
			int entityId = Integer.parseInt(str.substring(TAKEOFF.length()));
			((FlyingEntity) Battle.getInstance().getAllEntities().get(entityId))
					.doTakeOff();
		} else if (str.startsWith(LAND)) {
			int entityId = Integer.parseInt(str.substring(LAND.length()));
			((FlyingEntity) Battle.getInstance().getAllEntities().get(entityId))
					.doLand();
		} else if (str.startsWith(NEWORDER)) {
			int index1 = str.indexOf('#');
			int index2 = str.lastIndexOf('#');
			int armyId = Integer.parseInt(str.substring(NEWORDER.length(),
					index1));
			int groupId = Integer.parseInt(str.substring(index1 + 1, index2));
			char newOrder = str.substring(index2 + 1).charAt(0);
			((Command) Battle.getInstance().getArmies()[armyId].getCommands().get(groupId))
					.doChangeOrder(newOrder);
		} else if (str.startsWith(NEWUNITORDER)) {
			int index1 = str.indexOf('#');
			int entityId = Integer.parseInt(str.substring(
					NEWUNITORDER.length(), index1));
			char newOrder = str.substring(index1 + 1).charAt(0);

			Unit unit = ((Unit) Battle.getInstance().getAllEntities()
					.get(entityId));
			unit.doChangeOrder(newOrder);
		} else if (str.startsWith(CHAT)) {
			int index1 = str.indexOf('#');
			int index2 = index1 + 1 + str.substring(index1 + 1).indexOf('#');
			int clientId = Integer.parseInt(str
					.substring(CHAT.length(), index1));
			boolean toAll = str.substring(index1 + 1, index2).equals("1");
			String texte = str.substring(index2 + 1);
			FB.chat(clientId, toAll, texte);
		} else if (str.startsWith(SPENDMAGIC)) {
			int index = str.indexOf('#');
			int casterId = Integer.parseInt(str.substring(SPENDMAGIC.length(),
					index));
			int magicPoints = Integer.parseInt(str.substring(index + 1));
			((SpellCaster) Battle.getInstance().getAllEntities().get(casterId))
					.doSpendMagicPoints(magicPoints);
		} else if (str.startsWith(BANNISH)) {
			int entityId = Integer.parseInt(str.substring(BANNISH.length()));
			((SummonedCreature) Battle.getInstance().getAllEntities()
					.get(entityId)).doBannish();
		} else if (str.startsWith(COMMITSPELL)) {
			StringTokenizer paramTokenizer = new StringTokenizer(str
					.substring(COMMITSPELL.length()), "#");
			String spellName = paramTokenizer.nextToken();
			BasicSpell spell = null;
			if (spellName.equals("arcaneterror")) {
				spell = new Terror();
			} else if (spellName.equals("elimination")) {
				spell = new Lightnings();
			} else if (spellName.equals("resurection")) {
				spell = new Resurection();
			} else if (spellName.equals("fingerofdeath")) {
				spell = new EnergyBall();
			} else if (spellName.equals("transmission")) {
				spell = new Telepathy();
			} else if (spellName.equals("bannish")) {
				spell = new BannishDaemon();
			} else if (spellName.equals("shrink")) {
				spell = new Shrink();
			} else if (spellName.equals("speed")) {
				spell = new Speed();
			} else if (spellName.equals("heal")) {
				spell = new Heal();
			} else if (spellName.equals("magicshield")) {
				spell = new MagicShield();
			} else if (spellName.equals("projectileshield")) {
				spell = new ProjectileShield();
			} else if (spellName.equals("sunrise")) {
				spell = new Sunrise();
			} else if (spellName.equals("summon")) {
				spell = new SummonDaemon();
			}
			int casterId = Integer.parseInt(paramTokenizer.nextToken());
			SpellCaster caster = (SpellCaster) Battle.getInstance()
					.getAllEntities().get(casterId);
			spell.setCaster(caster);
			if (spell instanceof TargetSpell) {
				int targetId = Integer.parseInt(paramTokenizer.nextToken());
				((TargetSpell) spell).setTarget((Entity) Battle.getInstance()
						.getAllEntities().get(targetId));
			}
			if (spell instanceof ValuedSpell) {
				int value = Integer.parseInt(paramTokenizer.nextToken());
				((ValuedSpell) spell).setValue(value);
			}
			NetSpellInterruption phase = new NetSpellInterruption(spell);
			phase.init();
			phase.nextStep();
			phase.end();
		} else if (str.startsWith(BETRAY)) {
			int index1 = str.indexOf('#');
			int index2 = str.lastIndexOf('#');
			int armyId = Integer.parseInt(str
					.substring(BETRAY.length(), index1));
			int groupId = Integer.parseInt(str.substring(index1 + 1, index2));
			int ennemyArmyId = Integer.parseInt(str.substring(index2 + 1));
			((MercenaryCommand) Battle.getInstance().getArmies()[armyId].getCommands()
					.get(groupId))
					.doBetray(Battle.getInstance().getArmies()[ennemyArmyId]);
		} else if (str.startsWith(DEPLOYTO)) {
			int index1 = str.indexOf('#');
			int index2 = str.lastIndexOf('#');
			int entityId = Integer.parseInt(str.substring(DEPLOYTO.length(),
					index1));
			int x = Integer.parseInt(str.substring(index1 + 1, index2));
			int y = Integer.parseInt(str.substring(index2 + 1));
			((Entity) Battle.getInstance().getAllEntities().get(entityId))
					.doDeployTo(x, y);
		} else if (str.startsWith(YOURIDIS)) {
			int clientId = Integer.parseInt(str.substring(YOURIDIS.length()));
			Client.getInstance().setId(clientId);
			if (!FB.getInstance().isBatch()) {
				LocalPlayersPanel.getInstance().setNbPlayer(1);
			}
		} else if (str.startsWith(CONNECTION)) {
			int index1 = str.indexOf('#');
			String clientAdress = str.substring(CONNECTION.length(), index1);
			int clientId = Integer.parseInt(str.substring(index1 + 1));
			if (clientId != Client.getInstance().getClientId()) {
				ChatArea.getInstance().writeText(
						clientAdress + " " + Resources.getString("Connection"),
						FBColors.primary);
			}
			Client.getInstance().sendPlayers();

		} else if (str.startsWith(DISCONNECT)) {
			String clientAdress = str.substring(DISCONNECT.length());
			String message = clientAdress + " " + Resources.getString("Disconnection");
			ChatArea.getInstance().writeText(message, FBColors.primary);
			FB.getInstance().popUpMessage(clientAdress, message);
		} else if (str.startsWith(MYPLAYERSARE)) {
			int index1, index2, index3, index4;
			index1 = str.indexOf('#');
			index2 = index1 + 1 + str.substring(index1 + 1).indexOf('#');
			int clientId = Integer.parseInt(str.substring(
					MYPLAYERSARE.length(), index1));
			String clientName = str.substring(index1 + 1, index2);
			ChatArea.getInstance().addClientName(clientId, clientName);
			if (clientId < Client.getInstance().getClientId()
					&& !Client.getInstance().hasReceivedClientLists(clientId)) {
				// nouvel arrivant
				String playersList = str.substring(index2 + 1);
				for (int i = 0; !playersList.equals(""); i++) {
					index1 = playersList.indexOf('#');
					index2 = index1 + 1
							+ playersList.substring(index1 + 1).indexOf('#');
					index3 = index2 + 1
							+ playersList.substring(index2 + 1).indexOf('#');
					index4 = index3 + 1
							+ playersList.substring(index3 + 1).indexOf('#');
					String playerName = playersList.substring(0, index1);
					String playerArmy = playersList.substring(index1 + 1,
							index2);
					boolean playerIA = Boolean.valueOf(
							playersList.substring(index2 + 1, index3))
							.booleanValue();
					int playerSide = Integer.parseInt(playersList.substring(
							index3 + 1, index4));
					playersList = playersList.substring(index4 + 1);
					Battle.getInstance().getPlayersHandler().addNetPlayer(
							i + "@" + clientId, playerName, playerArmy,
							playerIA, playerSide);
				} // while
			} // if clientid
		} else if (str.startsWith(ADDPLAYER)) {
			int index1, index2, index3;
			index1 = str.indexOf('#');
			String playerId = str.substring(ADDPLAYER.length(), index1);
			int clientId = Integer.parseInt(playerId.substring(playerId
					.indexOf('@') + 1));
			if (clientId != Client.getInstance().getClientId()) {
				String player = str.substring(index1 + 1);
				index1 = player.indexOf('#');
				index2 = index1 + 1 + player.substring(index1 + 1).indexOf('#');
				index3 = index2 + 1 + player.substring(index2 + 1).indexOf('#');
				String playerName = player.substring(0, index1);
				String playerArmy = player.substring(index1 + 1, index2);
				boolean playerIA = Boolean.valueOf(
						player.substring(index2 + 1, index3)).booleanValue();
				int playerSide = Integer.parseInt(player.substring(index3 + 1));
				Battle.getInstance().getPlayersHandler().addNetPlayer(playerId,
						playerName, playerArmy, playerIA, playerSide);
			}
		} else if (str.startsWith(REMOVEPLAYER)) {
			String playerId = str.substring(REMOVEPLAYER.length());
			int clientId = Integer.parseInt(playerId.substring(playerId
					.indexOf('@') + 1));
			if (clientId != Client.getInstance().getClientId()) {
				Battle.getInstance().getPlayersHandler().removeNetPla(playerId);
			}
		} else if (str.startsWith(SETPLAYERNAME)) {
			int index1;
			index1 = str.indexOf('#');
			String playerId = str.substring(SETPLAYERNAME.length(), index1);
			int clientId = Integer.parseInt(playerId.substring(playerId
					.indexOf('@') + 1));
			if (clientId != Client.getInstance().getClientId()) {
				String playerName = str.substring(index1 + 1);
				Battle.getInstance().getPlayersHandler().setNetPlayerName(playerId,
						playerName);
			}
		} else if (str.startsWith(SETPLAYERARMY)) {
			int index1;
			index1 = str.indexOf('#');
			String playerId = str.substring(SETPLAYERARMY.length(), index1);
			int clientId = Integer.parseInt(playerId.substring(playerId
					.indexOf('@') + 1));
			if (clientId != Client.getInstance().getClientId()) {
				String playerArmy = str.substring(index1 + 1);
				Battle.getInstance().getPlayersHandler().setNetPlayerArmy(playerId,
						playerArmy);
			}
		} else if (str.startsWith(SETPLAYERSIDE)) {
			int index1;
			index1 = str.indexOf('#');
			String playerId = str.substring(SETPLAYERSIDE.length(), index1);
			int clientId = Integer.parseInt(playerId.substring(playerId
					.indexOf('@') + 1));
			if (clientId != Client.getInstance().getClientId()) {
				int playerSide = Integer.parseInt(str.substring(index1 + 1));
				Battle.getInstance().getPlayersHandler().setNetPlayerSide(playerId,
						playerSide);
			}
		} else if (str.startsWith(SETPLAYERAI)) {
			int index1;
			index1 = str.indexOf('#');
			String playerId = str.substring(SETPLAYERAI.length(), index1);
			int clientId = Integer.parseInt(playerId.substring(playerId
					.indexOf('@') + 1));
			if (clientId != Client.getInstance().getClientId()) {
				boolean playerAI = Boolean.valueOf(str.substring(index1 + 1))
						.booleanValue();
				Battle.getInstance().getPlayersHandler().setNetPlaIA(playerId,
						playerAI);
			}
		} else if (str.startsWith(SETMAP)) {
			if (!Server.hasInstance()) {
				String mapName = str.substring(SETMAP.length());
				MapChooser.getInstance(false).setMap(mapName);
			}
		}
	}
}