/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.phases;

import java.util.List;

import fb.army.entities.Entity;
import fb.artificialintelligence.TargetFinder;
import fb.gui.frames.BattleFrame;
import fb.main.Battle;

/**
 * game phase played at any time during battle on player s request that
 * interrupts current phase
 * 
 * @author farfpeuf
 * 
 */
public abstract class InterruptingPhase extends BinaryPhase {

	/**
	 * interrupted phase
	 */
	private BinaryPhase interrupted;

	/**
	 * token used to know if the player army has finished this phase
	 */
	private boolean token = false;

	/**
	 * constructor interrupt current phase
	 */
	public InterruptingPhase() {
		super();
		interrupted = (BinaryPhase) Battle.getLoop().currentSynchronizedPhase;
		interrupted.interrupt();
		current = interrupted.current;
	}

	/**
	 * change current playing army
	 * 
	 */
	public final void changeArmy() {
		// only current player s army plays this phase
	}

	/**
	 * has this entity targets for this phase ?
	 * 
	 * @param actor
	 *            the phase actor
	 * @return true if this entity has targets for this phase ?
	 */
	public final boolean hasTargets(Entity actor) {
		Entity entity;
		List entities = armies[current].getActiveEntities();
		for (int i = 0; i < entities.size(); i++) {
			entity = (Entity) (entities.get(i));
			if (entity.isActive()) {
				if (isTarget(actor, entity)) {
					return true;
				}
			}
		}
		return false;

	}

	/**
	 * wait for player action
	 * 
	 */
	public final void waitForPlayerAction() {
		BattleFrame.getInstance().getNextPhaseButton().setEnabled(true);
		if (isActorSelection()) {
			setDefaultCursor();
		} else {
			setTargetCursor();
		}
		waitingForPlayerAction = true;
	}

	/**
	 * player Performed an Action
	 * 
	 */
	public final void playerPerformedAction() {
		setWaitCursor();
		BattleFrame.getInstance().getNextPhaseButton().setEnabled(false);
		if (clickedPoint == null) { // nextphase
			token = true;
		} else {
			handlePlayerAction();
		}
	}

	/**
	 * proceed to phase next step
	 */
	public final void nextStep() {
		waitForPlayerAction();
	}

	/**
	 * determine first playing army
	 * 
	 */
	public final void initCurrent() {
		Battle.getInstance().setArmy(armies[current]);
		// FB.debug("initCurrent " + armies[current].getPlayerName());
	}

	/**
	 * set token to specify that an army hass finished to play phase
	 * 
	 * @param armyId
	 *            the army index
	 */
	public final void setToken(int armyId) {
		token = true;
	}

	/**
	 * @return true if phase is finished
	 */
	public final boolean finished() {
		return token;
	}

	/**
	 * @return true if all the armies handled by this computer finished this
	 *         phase
	 */
	public boolean allHandledArmiesDone() {
		return token;
	}

	/**
	 * @return AI thread that handles choice of the best target among a set of
	 *         availables targets
	 */
	public TargetFinder getTargetFinder() {
		return null;// not used

	}

	/**
	 * @return Returns the interrupted.
	 */
	public BinaryPhase getInterrupted() {
		return interrupted;
	}

}
