/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.phases;

import fb.gui.frames.BattleFrame;
import fb.main.FB;
import fb.main.Battle;
import fb.spells.BasicSpell;

/**
 * phase to handle spells casted by remote players
 * 
 * @author Farfpeuf
 */
public class NetSpellInterruption extends Phase {
	/**
	 * phase name
	 */
	private static final String name = "netmagicphase";

	/**
	 * phase icon number
	 */
	private static final int iconNumber = 5;

	/**
	 * casted spell
	 */
	private BasicSpell spell;

	/**
	 * interrupted phase
	 */
	private BinaryPhase interrupted;

	/**
	 * is phase finished ?
	 */
	private boolean finished;

	/**
	 * constructor
	 * 
	 * @param spell
	 *            the casted spell
	 */
	public NetSpellInterruption(BasicSpell spell) {
		this.spell = spell;
	}

	/**
	 * @return phase name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @return phase icon number
	 */
	public int getIconNumber() {
		return iconNumber;
	}

	/**
	 * @return phase toolip
	 */
	public String getTooltip() {
		return null;// no tooltip
	}

	/**
	 * initialize this phase
	 * 
	 */
	public void init() {
		finished = false;
	}

	/**
	 * @return true if phase is finished
	 */
	public boolean finished() {
		return finished;
	}

	/**
	 * proceed to phase next step
	 */
	public void nextStep() {
		BattleFrame.getInstance().setPhase(this);
		// block clics
		interrupted = Battle.getLoop().interruptCurrentPhase();

		Thread thread = new Thread() {
			public void run() {
				setName("Net magic interruption thread");
				int interruptedcursor = FB.getInstance().getCurrentCursor();
				FB.setWaitCursor();
				spell.doCommitSpell();
				Battle.getLoop().notifyAndResume();
				FB.getInstance().setBattlefieldCursor(interruptedcursor);
				if (interrupted.armies[interrupted.current].isHandled()
						&& !interrupted.armies[interrupted.current].isAi()) {
					BattleFrame.getInstance().enableButtons(true);
				} // if
				if (Battle.getLoop().currentSynchronizedPhase == interrupted) {
					BattleFrame.getInstance().setPhase(interrupted);
				}
				finished = true;
			} // run
		}; // thread
		thread.start();

	}

	/**
	 * end this phase
	 */
	public void end() {
		// nothing to do
	}
}