/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.phases;

import java.util.List;

import fb.army.entities.Courier;
import fb.army.entities.Entity;
import fb.army.entities.Leader;
import fb.army.entities.MessageReceiver;
import fb.army.entities.Unit;
import fb.gui.frames.BattleFrame;
import fb.main.FB;
import fb.main.Battle;
import fb.util.Resources;
import fb.util.StringTools;

/**
 * order phase for human players
 * 
 * @author farfpeuf
 * 
 */
public class PlayerOrderPhase extends InterruptingPhase {
	/**
	 * phase name
	 */
	private static final String name = "newordersphase";

	/**
	 * phase icon number
	 */
	private static final int iconNumber = 4;

	/**
	 * selected new order
	 */
	public char newOrder;

	/**
	 * construcor
	 * 
	 */
	public PlayerOrderPhase() {
		super();
		setWithPhasePanel(true);
	}

	/**
	 * @return phase name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @return phase icon number
	 */
	public int getIconNumber() {
		return iconNumber;
	}

	/**
	 * calculate available targets
	 * 
	 */
	public void calculatePotentialTargets() {

		FB.setWaitCursor();
		Entity entity;
		clearPotentialTargets();
		List entities = armies[current].getActiveEntities();
		for (int i = 0; i < entities.size(); i++) {
			entity = (Entity) (entities.get(i));
			if (entity.isActive()) {
				if (isTarget(entity)) {
					addPotentialTarget(entity);
				}
			}

		}

		if (!armies[current].isAi()) {
			FB.getInstance().setBattlefieldCursor(FB.TARGET_CURSOR);
		}
		setActorselection(false);
	}

	/**
	 * initialize actor selection on the battlefield
	 * 
	 */
	public void initActorSelection() {
		if (!finished()) {
			super.initActorSelection();
			if (getPotentialActors().size() == 1) {
				setActor((Entity) getPotentialActors().get(0));
				initTargetSelection();
			}
		}
	}

	/**
	 * is this entity a potential target for current actor
	 * 
	 * @param e
	 *            phase target
	 * @return true if this entity is a potential target for current actor
	 */
	public boolean isTarget(Entity entity) {
		if (entity.isMessageReceiver() && !entity.isEnnemy(getActor())) {
			MessageReceiver receiver = (MessageReceiver) entity;
			if (receiver.getNewOrder() == 'n') {
				if (getActor().isWarchief()) {
					if (receiver.isWarchief()) {
						Leader l = (Leader) receiver;
						if (l.orderDistanceOkForAll()
								&& !l.getCommand().hasChangedOrders()
								&& l.getCommand().getUnits().size() != 0) {
							return true;
						}
					} else if (receiver.nearWarchief()
							&& ((receiver.getCommand().getUnits().size() != 0 && !receiver
									.getCommand().hasChangedOrders()) || receiver
									.isCourier())) {
						return true;
					}
				} else {
					if (receiver.isLeader() && !receiver.isWarchief()) {
						return true;
					}
				}
			}
		} else if (entity.isUnit()) {
			Unit u = (Unit) entity;
			if (!u.isChangedOrders()) {

				if (getActor().isLeader()) {
					Leader l = (Leader) getActor();
					if (l.getCommand() == entity.getCommand()
							&& l.stepsTo(entity) < l.orderRange()) {
						return true;
					}
				} else {
					Courier courier = (Courier) getActor();
					if (courier.getSender() == entity.getCommand().getLeader()) {
						return true;
					}
				}
			}
		}
		return false;
	}

	/**
	 * is this entity a potential target for specified actor
	 * 
	 * @param a
	 *            phase actor
	 * @param e
	 *            phase target
	 * @return true if this entity is a potential target for specified actor
	 */
	public boolean isTarget(Entity actor, Entity entity) {
		if (entity.isMessageReceiver() && !entity.isEnnemy(actor)) {
			MessageReceiver receiver = (MessageReceiver) entity;
			if (receiver.getNewOrder() == 'n') {
				if (receiver.isWarchief()) {
					Leader l = (Leader) receiver;
					if (l.orderDistanceOkForAll()
							&& !l.getCommand().hasChangedOrders()
							&& l.getCommand().getUnits().size() != 0) {

						return true;
					}
				} else if (receiver.nearWarchief()
						&& ((receiver.getCommand().getUnits().size() != 0 && !receiver
								.getCommand().hasChangedOrders()) || receiver
								.isCourier())) {
					return true;
				}
			}
		} else if (entity.isUnit()) {
			Unit u = (Unit) entity;
			if (!u.isChangedOrders()) {
				if (actor.isLeader()) {
					Leader l = (Leader) actor;
					if (l.getCommand() == entity.getCommand()
							&& l.stepsTo(entity) < l.orderRange()) {
						return true;
					}
				} else {
					Courier courier = (Courier) actor;
					if (courier.getSender() == entity.getCommand().getLeader()) {
						return true;
					}
				}
			}
		}
		return false;
	}

	/**
	 * is this entity a potential actor for this phase
	 * 
	 * @param a
	 *            phase actor
	 * @return true if this entity is a potential actor
	 */
	public boolean isActor(Entity e) {
		return e.isLeader();
	}

	/**
	 * handle human player action
	 * 
	 */
	public void handlePlayerAction() {
		Entity entity;
		boolean entityselected = false;

		List selectables;

		if (isActorSelection()) {
			selectables = getPotentialActors();
		} else {
			selectables = getPotentialTargets();
		}

		for (int i = 0; i < selectables.size(); i++) {
			entity = (Entity) (selectables.get(i));
			if (entity.getArea().contains(clickedPoint)) {
				entityselected = true;
				selected = entity;
			}
		}

		if (entityselected) {
			BattleFrame.getInstance().displayEntityCharacteristics(selected);

			if (isActorSelection()) {
				setActor(selected);
				initTargetSelection();

			} else {
				setTarget(selected);

				if (getTarget().isCourier()) {
					Courier courier = (Courier) getTarget();
					courier.setSender((Leader) getActor());
					setActor(getTarget());
					initTargetSelection();
				} else {
					initActorSelection();
					BattleFrame.getInstance().setOrderPanel(this);
				}
			}
		} else {
			initActorSelection();
		}
	}

	/**
	 * calculate phase results
	 * 
	 */
	public void calculateResults() {

		Entity entity;

		for (int i = 0; i < Battle.getInstance().getActiveEntities().size(); i++) {
			entity = (Entity) (Battle.getInstance().getActiveEntities().get(i));
			if (entity.isCourier()) {
				Courier courier = (Courier) entity;
				Entity receiver = courier.getReceiver();
				if (courier.getNewOrder() != 'n') {
					if (courier.near(receiver)) {

						if (receiver.isLeader()) {
							((Leader) receiver).setNewOrder(courier
									.getNewOrder());
							((Leader) receiver).setMessenger(courier);
						} else {
							((Unit) receiver)
									.changeOrder(courier.getNewOrder());
							courier.setReceiver(null);
						}
						String message = Resources.getString("couriertransmit");
						message = StringTools.replaceAll(message,
								"<entity full name>", courier.getFullName());
						FB.getInstance().message(message, courier.getColor());
					}

				}
			} // couriers
			else if (entity.isLeader()) {
				Leader leader = (Leader) entity;
				if (leader.getNewOrder() != 'n') {
					if (leader.orderDistanceOkForAll()) {
						if (leader.isWarchief()) {
							leader.getCommand().changeOrder(
									leader.getNewOrder());
						} else if (leader.accceptNewOrders()) {
							if (leader.getMessenger().isCourier()) {
								Courier courier = (Courier) leader
										.getMessenger();
								courier.setNewOrder('n');
								courier.setReceiver(null);
							}

						}
					} else {
						String message = Resources.getString("toofar");
						message = StringTools.replaceAll(message,
								"<entity full name>", leader.getFullName());
						FB.getInstance().message(message, leader.getColor());
					}
					leader.setNewOrder('n');
					leader.setMessenger(null);
				}
			} // leaders

		}

	}

	/**
	 * set new order
	 * 
	 * @param o
	 *            the new order
	 */
	public synchronized void setOrder(char o) {
		if (waitingForPlayerAction) {
			waitingForPlayerAction = false;
			newOrder = o;
			BattleFrame.getInstance().clearActionPanel();
			if (getActor().isLeader()) {
				Entity receiver = getTarget();
				if (receiver.isLeader()) {
					((Leader) receiver).setNewOrder(newOrder);
					((Leader) receiver).setMessenger((Leader) getActor());
				} else {
					((Unit) receiver).changeOrder(newOrder);
				}

			} else {
				Courier courier = (Courier) getActor();
				courier.setNewOrder(newOrder);
				courier.setReceiver(getTarget());

				String orderstring;
				if (newOrder == 'a') {
					orderstring = "attack";
				} else if (newOrder == 'o') {
					orderstring = "oppose";
				} else {
					orderstring = "hold";
				}
				String message = Resources.getString("couriernew");
				message = StringTools.replaceAll(StringTools.replaceAll(
						message, "<entity full name>", courier.getFullName()),
						"<order name>", Resources.getString(orderstring));
				FB.getInstance().message(message, courier.getColor());
			}
			Battle.getLoop().resume();
		}
	}

}
