/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.player;

import java.util.Hashtable;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import fb.main.FB;
import fb.network.Client;
import fb.network.MessagesConstants;

/**
 * players manager
 * 
 * @author farfpeuf
 * 
 */
public class PlayersHandler implements TableModelListener {
	/**
	 * local players table data model
	 */
	private PlayersTableModel localPlayersTableModel = new PlayersTableModel(
			true);

	/**
	 * network players table data model
	 */
	private PlayersTableModel netPlayersTableModel = new PlayersTableModel(
			false);

	/**
	 * network players id to table row map
	 */
	private Hashtable netPlayersIToRowMap = new Hashtable();

	/**
	 * @return total number of players
	 */
	public int getTotalNbPlayers() {
		return localPlayersTableModel.getNbPlayer()
				+ netPlayersTableModel.getNbPlayer();
	}

	/**
	 * constructor register to local Players Table Model
	 */
	public PlayersHandler() {
		localPlayersTableModel.addTableModelListener(this);
	}

	/**
	 * add a network player
	 * 
	 * @param id
	 *            the player id
	 * @param name
	 *            the player name
	 * @param army
	 *            the player army
	 * @param isAi
	 *            human player or artificial intelligence?
	 * @param side
	 *            the player side
	 */
	public void addNetPlayer(String id, String name, String army, boolean isAi,
			int side) {

		netPlayersTableModel.addPlayer(name, army, isAi, side);
		netPlayersIToRowMap.put(id, new Integer(
				netPlayersTableModel.getNbPlayer() - 1));

	}

	/**
	 * remove a network player
	 * 
	 * @param id
	 *            the player id
	 */
	public void removeNetPla(String id) {
		Integer row = (Integer) netPlayersIToRowMap.remove(id);
		if (row != null) {
			netPlayersTableModel.removePlayer(row.intValue());
		}

	}

	/**
	 * change network player side
	 * 
	 * @param id
	 *            the player id
	 * @param side
	 *            the new side
	 */
	public void setNetPlayerSide(String id, int side) {
		Integer row = (Integer) netPlayersIToRowMap.get(id);
		if (row != null) {
			netPlayersTableModel.setPlayerSide(row.intValue(), side);
		}
	}

	/**
	 * change network player name
	 * 
	 * @param id
	 *            the player id
	 * @param name
	 *            the new name
	 */
	public void setNetPlayerName(String id, String name) {
		Integer row = (Integer) netPlayersIToRowMap.get(id);
		if (row != null) {
			netPlayersTableModel.setPlayerName(row.intValue(), name);
		}
	}

	/**
	 * change network player army
	 * 
	 * @param id
	 *            the player id
	 * @param army
	 *            the new army
	 */
	public void setNetPlayerArmy(String id, String army) {
		Integer row = (Integer) netPlayersIToRowMap.get(id);
		if (row != null) {
			netPlayersTableModel.setPlayerArmy(row.intValue(), army);
		}

	}

	/**
	 * change network player ai attribute
	 * 
	 * @param id
	 *            the player id
	 * @param ai
	 *            the new ai attribute
	 */
	public void setNetPlaIA(String id, boolean ai) {
		Integer row = (Integer) netPlayersIToRowMap.get(id);
		if (row != null) {
			netPlayersTableModel.setPlayerAI(row.intValue(), ai);
		}
	}

	/**
	 * action when players table data changed
	 */
	public void tableChanged(TableModelEvent e) {
		if (FB.isNet()) {
			int row = e.getFirstRow();
			switch (e.getType()) {
			case TableModelEvent.INSERT:
				Player pla = localPlayersTableModel.getPla(row);
				Client.getInstance().send(
						MessagesConstants.ADDPLAYER + row + "@"
								+ Client.getInstance().getClientId() + "#"
								+ pla.getName() + "#" + pla.getArmyName() + "#"
								+ pla.isIa() + "#" + pla.getSide());
				break;
			case TableModelEvent.DELETE:
				Client.getInstance().send(
						MessagesConstants.REMOVEPLAYER + row + "@"
								+ Client.getInstance().getClientId());
				break;
			case TableModelEvent.UPDATE:

				int column = e.getColumn();
				pla = localPlayersTableModel.getPla(row);
				if (column == 0) {
					Client.getInstance().send(
							MessagesConstants.SETPLAYERNAME + row + "@"
									+ Client.getInstance().getClientId() + "#"
									+ pla.getName());
				} else if (column == 1) {
					Client.getInstance().send(
							MessagesConstants.SETPLAYERAI + row + "@"
									+ Client.getInstance().getClientId() + "#"
									+ pla.isIa());
				} else if (column == 2) {
					Client.getInstance().send(
							MessagesConstants.SETPLAYERARMY + row + "@"
									+ Client.getInstance().getClientId() + "#"
									+ pla.getArmyName());
				} else if (column == 3) {
					Client.getInstance().send(
							MessagesConstants.SETPLAYERSIDE + row + "@"
									+ Client.getInstance().getClientId() + "#"
									+ pla.getSide());
				}
				break;

			}
		}

	}

	/**
	 * @return the local Players Table Model
	 */
	public PlayersTableModel getLocalPlayersTableModel() {
		return localPlayersTableModel;
	}

	/**
	 * @return the network Players Table Model
	 */
	public PlayersTableModel getNetPlayersTableModel() {
		return netPlayersTableModel;
	}

}
