package fb.sound;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.InputStream;

import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;

import com.jcraft.jogg.Packet;
import com.jcraft.jogg.Page;
import com.jcraft.jogg.StreamState;
import com.jcraft.jogg.SyncState;
import com.jcraft.jorbis.Block;
import com.jcraft.jorbis.Comment;
import com.jcraft.jorbis.DspState;
import com.jcraft.jorbis.Info;

/**
 * an ogg sound file decoder
 * 
 * 
 */
public class OggDecoder {

	//constant
	
	private static final int BUFF_SIZE = 8192;

	private int size = BUFF_SIZE * 2;

	private byte[] buffy = new byte[size];

	private ByteArrayOutputStream outputStream = null;

	private AudioFormat format;
	
	private byte[] buffer = null;

	private int nbBytesRead = 0;

	//jorbis objects
	
	
	private Block block;
	
	private Page page;

	private Packet packet;

	private Info info;

	private Comment comment;

	private DspState dspState;

	private SyncState syncState;

	private StreamState streamState;


	/**
	 * decode ogg stream into file
	 * @param sourceStream ogg source strem
	 * @param outputFile destination file
	 * @throws Exception
	 */
	
	public void decode(InputStream sourceStream, File outputFile) throws Exception {

		int nbSamples = 0;

		boolean ch = false;

		syncState = new SyncState();
		streamState = new StreamState();
		page = new Page();
		packet = new Packet();

		info = new Info();
		comment = new Comment();
		dspState = new DspState();
		block = new Block(dspState);

		buffer = null;
		nbBytesRead = 0;

		syncState.init();

		main: while (true) {
			int endOfStream = 0;

			int offset = syncState.buffer(BUFF_SIZE);
			buffer = syncState.data;
			nbBytesRead = sourceStream.read(buffer, offset, BUFF_SIZE);

			syncState.wrote(nbBytesRead);

			if (ch) { 
				ch = false;  
			} 
			else { 
				if (syncState.pageout(page) != 1) {
					if (nbBytesRead < BUFF_SIZE)
						break;

				}
			} 
			streamState.init(page.serialno());
			streamState.reset();

			info.init();
			comment.init();

			streamState.pagein(page);

			streamState.packetout(packet);

			info.synthesis_headerin(comment, packet);


			int index = 0;

			while (index < 2) {
				while (index < 2) {
					int result = syncState.pageout(page);
					if (result == 0)
						break; 
					if (result == 1) {
						streamState.pagein(page);
						while (index < 2) {
							result = streamState.packetout(packet);
							if (result == 0)
								break;
							if (result == -1) {
								System.err
										.println("Corrupt secondary header.  Exiting.");
								
								break main;
							}
							info.synthesis_headerin(comment, packet);
							index++;
						}
					}
				}

				offset = syncState.buffer(BUFF_SIZE);
				buffer = syncState.data;

				nbBytesRead = sourceStream.read(buffer, offset, BUFF_SIZE);


				syncState.wrote(nbBytesRead);
			}

			size = BUFF_SIZE / info.channels;

			dspState.synthesis_init(info);
			block.init(dspState);

			float[][][] data3D = new float[1][][];
			int[] indexes = new int[info.channels];

			format = new AudioFormat((float) info.rate, 16, info.channels,
					true, false);
			outputStream = new ByteArrayOutputStream();

			while (endOfStream == 0) {
				while (endOfStream == 0) {

					int result = syncState.pageout(page);
					if (result == 0)
						break;
					if (result == -1) {
					} else {
						streamState.pagein(page);

						if (page.granulepos() == 0) {
							ch = true;
							endOfStream = 1;
							break;
						}

						while (true) {
							result = streamState.packetout(packet);
							if (result == 0)
								break;
							if (result == -1) {
							} else {

								int samples;
								if (block.synthesis(packet) == 0) {

									dspState.synthesis_blockin(block);
								}
								while ((samples = dspState.synthesis_pcmout(
										data3D, indexes)) > 0) {
									nbSamples += samples;
									float[][] data2D = data3D[0];

									int b = (samples < size ? samples
											: size);

									for (index = 0; index < info.channels; index++) {
										int index2 = index * 2;

										int mono = indexes[index];
										for (int j = 0; j < b; j++) {
											int val = (int) (data2D[index][mono + j] * 32767.);
											if (val > 32767) {
												val = 32767;
											}
											if (val < -32768) {
												val = -32768;
											}
											if (val < 0)
												val = val | 0x8000;
											buffy[index2] = (byte) (val);
											buffy[index2 + 1] = (byte) (val >>> 8);
											index2 += 2 * (info.channels);
										}
									}
									outputStream.write(buffy, 0, 2 * info.channels
											* b);
									dspState.synthesis_read(b);
								}
							}
						}
						if (page.eos() != 0)
							endOfStream = 1;
					}
				}

				if (endOfStream == 0) {
					offset = syncState.buffer(BUFF_SIZE);
					buffer = syncState.data;
					try {
						nbBytesRead = sourceStream.read(buffer, offset, BUFF_SIZE);
					} catch (Exception e) {
						System.err.println(e);
						return;
					}
					if (nbBytesRead == -1) {
						break;
					}
					syncState.wrote(nbBytesRead);
					if (nbBytesRead == 0)
						endOfStream = 1;
				}
			}

			streamState.clear();
			block.clear();
			dspState.clear();
			info.clear();
		}

		syncState.clear();

		try {
			if (sourceStream != null)
				sourceStream.close();
		} catch (Exception e) {
		}
		byte[] bytes = outputStream.toByteArray();
		ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
		AudioInputStream ais = new AudioInputStream(bais, format,
				nbSamples);
		AudioSystem.write(ais, AudioFileFormat.Type.WAVE, outputFile);

	}

}
