/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.spells;

import fb.army.entities.Entity;
import fb.army.entities.Heroic;
import fb.army.entities.Individual;
import fb.army.entities.Leader;
import fb.army.entities.SpellCaster;
import fb.army.entities.SummonedCreature;
import fb.util.D6;

/**
 * bring a dead individual back to life
 * 
 * @author farfpeuf
 * 
 */
public class Resurection extends TargetSpell {
	/**
	 * number of die to roll
	 */
	private static final int dieToRoll = 5;

	/**
	 * constructor
	 * 
	 */
	public Resurection() {
		super();
		name = "resurection";
	}

	/**
	 * is this entity a potential target for specified actor
	 * 
	 * @param a
	 *            spell actor
	 * @param e
	 *            spell target
	 * @return true if this entity is a potential target for specified actor
	 */
	public boolean isTarget(Entity a, Entity e) {
		return e instanceof Individual && !(e instanceof SummonedCreature)
				&& !e.isEnnemy(a) && e.getState() == 'a'
				&& super.isTarget(a, e);
	}

	/**
	 * is this entity a potential target for current actor
	 * 
	 * @param e
	 *            spell target
	 * @return true if this entity is a potential target for current spell
	 *         caster
	 */
	public boolean isTarget(Entity e) {
		return e instanceof Individual && !(e instanceof SummonedCreature)
				&& !e.isEnnemy(caster) && e.getState() == 'a'
				&& super.isTarget(e);
	}

	/**
	 * cast the spell
	 * 
	 * @return true if the spell was successfull
	 */
	public boolean castSpell() {

		if (caster.spendMagicPoints(cost())) {

			return true;
		}
		return false;
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell cost
	 */
	public int cost(SpellCaster caster, Entity target) {
		int cost = 0;
		for (int i = 0; i < dieToRoll; i++) {
			cost += D6.roll();
		}
		return cost;
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell maximum cost
	 */
	public int maxCost(SpellCaster caster, Entity target) {
		return dieToRoll * 6;
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell minimum cost
	 */
	public int minCost(SpellCaster caster, Entity target) {
		return dieToRoll * 1;
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return average cost
	 */
	public int averageCost(SpellCaster caster, Entity target) {
		return (int) ((double) dieToRoll * 3.5);
	}

	/**
	 * end the spell
	 * 
	 * @throws InterruptedException
	 */
	public void endSpell() throws InterruptedException {
		((Individual) target).resurection(caster);
		magicSynchronize();
	}

	/**
	 * @param potentialTarget
	 *            the target
	 * @return weight modifiers for artificial intelligence for specified target
	 */
	public int getWeight(Entity potentialTarget) {
		int weight = super.getWeight(potentialTarget);

		if (potentialTarget instanceof Heroic
				|| (potentialTarget instanceof Leader && !((Leader) potentialTarget)
						.getCommand().isRoutedAnnihilated())) {
			weight += 3;
		}

		return weight;
	}
}