/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.spells;

import fb.army.entities.Entity;
import fb.army.entities.Heroic;
import fb.army.entities.SpellCaster;
import fb.army.entities.Unit;
import fb.util.D6;

/**
 * Increases by 50% the movement of the friendly target during 3 turns
 * 
 * @author farfpeuf
 * 
 */
public class Speed extends TargetSpell {
	/**
	 * constructor
	 * 
	 */
	public Speed() {
		super();
		name = "speed";
	}

	/**
	 * is this entity a potential target for specified actor
	 * 
	 * @param a
	 *            spell actor
	 * @param e
	 *            spell target
	 * @return true if this entity is a potential target for specified actor
	 */
	public boolean isTarget(Entity a, Entity e) {
		return e.getHaste() == 0 && e.getOpposingContacts().size() == 0
				&& !e.isEnnemy(a) && a.lineOfSightTo(e) < 9999
				&& super.isTarget(a, e);
	}

	/**
	 * is this entity a potential target for current actor
	 * 
	 * @param e
	 *            spell target
	 * @return true if this entity is a potential target for current spell
	 *         caster
	 */
	public boolean isTarget(Entity e) {
		return e.getHaste() == 0 && e.getOpposingContacts().size() == 0
				&& !e.isEnnemy(caster) && caster.lineOfSightTo(e) < 9999
				&& super.isTarget(e);
	}

	/**
	 * cast the spell
	 * 
	 * @return true if the spell was successfull
	 */
	public boolean castSpell() {

		if (caster.spendMagicPoints(cost())) {

			return true;
		}
		return false;
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell cost
	 */
	public int cost(SpellCaster caster, Entity target) {
		return (int) (0.999 + caster.lineOfSightTo(target) / 12) + D6.roll()
				+ D6.roll();

	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell maximum cost
	 */
	public int maxCost(SpellCaster caster, Entity target) {
		return 12 + (int) (caster.lineOfSightTo(target) / 12);
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell minimum cost
	 */
	public int minCost(SpellCaster caster, Entity target) {
		return 2 + (int) (caster.lineOfSightTo(target) / 12);
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return average cost
	 */
	public int averageCost(SpellCaster caster, Entity target) {
		return (int) (7 + (int) (caster.lineOfSightTo(target) / 12));
	}

	/**
	 * end the spell
	 * 
	 * @throws InterruptedException
	 */
	public void endSpell() throws InterruptedException {
		target.initHaste();
		magicSynchronize();
	}

	/**
	 * @param potentialTarget
	 *            the target
	 * @return weight modifiers for artificial intelligence for specified target
	 */
	public int getWeight(Entity potentialTarget) {
		int weight = super.getWeight(potentialTarget);
		if (potentialTarget instanceof Heroic
				|| potentialTarget instanceof Unit) {
			weight += 2;
		}

		if (potentialTarget.getMovement() < 7) {
			weight += 1;
		}
		return weight;
	}
}