/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.util;

import java.io.File;
import java.io.FileFilter;
import java.util.Locale;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;

import fb.main.Options;

/**
 * fantasy battlefields installer
 * 
 * @author farfpeuf
 * 
 */
public class GameInstaller {
	/**
	 * localisation file name prefix
	 */
	private static final String messageFileName = "Messages";

	/**
	 * localisation file name pattern
	 */
	private static final String messageFilePattern = messageFileName
			+ "_[a-z][a-z]_[A-Z][A-Z]\\.properties";

	/**
	 * available locales
	 */
	private static Locale[] availableLocales;

	/**
	 * available languages
	 */
	private static String[] availableLanguages;

	/**
	 * constructor build option pane and show it
	 */
	public GameInstaller() {
		int rep = JOptionPane.showOptionDialog(null, "Choose your language.",
				"Fantasy Battlefields", JOptionPane.OK_OPTION,
				JOptionPane.QUESTION_MESSAGE, null, getAvailableLanguages(),
				getLocaleName(Locale.getDefault()));
		if (rep == JOptionPane.CLOSED_OPTION) {
			System.exit(0);
		} else {
			Locale locale = getAvailableLocales()[rep];

			Options.setLanguage(locale.getLanguage());
			Options.setCountry(locale.getCountry());

			Options.getInstance().save();
		}
	}

	/**
	 * @return available locales for the game
	 */
	public static synchronized Locale[] getAvailableLocales() {
		if (availableLocales == null) {
			File root = new File(".");
			File[] messagesFiles = root.listFiles(new FileFilter() {
				public boolean accept(File pathname) {
					return pathname.isFile()
							&& Pattern.matches(messageFilePattern, pathname
									.getName());
				}
			});

			availableLocales = new Locale[messagesFiles.length];
			String localePart;
			String[] tokens;

			for (int i = 0; i < messagesFiles.length; i++) {
				localePart = messagesFiles[i].getName().substring(
						messageFileName.length() + 1,
						messageFileName.length() + 6);
				tokens = localePart.split("_");
				availableLocales[i] = new Locale(tokens[0], tokens[1]);
			}

		}
		return availableLocales;
	}

	/**
	 * @return available languages for the game
	 */
	public static synchronized String[] getAvailableLanguages() {
		if (availableLanguages == null) {
			Locale locale;
			availableLanguages = new String[getAvailableLocales().length];

			for (int i = 0; i < getAvailableLocales().length; i++) {
				locale = getAvailableLocales()[i];
				availableLanguages[i] = getLocaleName(locale);
			}

		}
		return availableLanguages;
	}

	/**
	 * get locale display name
	 * 
	 * @param locale
	 *            the locale
	 * @return the loacle display name
	 */
	public static String getLocaleName(Locale locale) {
		String name = locale.getDisplayLanguage(locale);
		if (name.equals("")) {
			name = locale.getLanguage();
		}
		return name;
	}

}